# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Mtrafld(NysolMOD_CORE):

	_kwd,_inkwd,_outkwd = n_core.getparalist("mtrafld",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_Mtrafld,self).__init__("mtrafld",nutil.args2dict(args,kw_args,Nysol_Mtrafld._kwd))

def mtrafld(self,*args, **kw_args):
	return Nysol_Mtrafld(nutil.args2dict(args,kw_args,Nysol_Mtrafld._kwd)).addPre(self)

setattr(NysolMOD_CORE, "mtrafld", mtrafld)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import os.path
import shutil

import nysol.mcmd as nm
import nysol.util as nu
from nysol.take import extcore as extTake


class mbipolish(object):

	helpMSG="""
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) 2部グラフの研磨をおこなう。
内容) 2部グラフを入力として、密度の高い2部部分グラフにあって枝が張られていないノードペアに枝を張る。
      逆に、密度の低い2部部分グラフにあって枝が張られているノードペアの枝を刈る。
      新たに張られる枝や刈られる枝の程度は、sim=,th=とsim2,th2で与えた値によって変わる。
書式) #{$cmd} ei= [ef=] [nf=] eo= [sim=i|I|s|S|T|R|P|C] th= [th2=] [sim2=i|I|s|S|T|R|P|C] [sup=] [iter=] [log=] [T=] [--help]

  ファイル名指定
  ei=    : 枝データファイル
  ef=    : 枝データ上の2つの節点項目名(省略時は"node1,node2")
  eo=    : データ研磨後の枝データファイル
  sim|2= : 節点a,bと接続された枝集合を、それぞれA,Bとすると、節点a,bに枝を張るために用いる類似度。
           省略時はRが設定される。(sim2のデフォルト:sim=)
             i: inclusion
             I: both-inclusion
             S: |A∩B|/max(|A|,|B|)
             s: |A∩B|/min(|A|,|B|)
             T (intersection): find pairs having common [threshld] items
             R (resemblance): find pairs s.t. |A\capB|/|A\cupB| >= [threshld]
             P (PMI): find pairs s.t. log (|A\capB|*|all| / (|A|*|B|)) >= [threshld]
             C (cosine distance): find pairs s.t. inner product of their normalized vectors >= [threshld]
  th|2=  : sim|2=で指定された類似度について、ここで指定された値以上の節点間に枝を張る。(th2のデフォルト:th=)
  sup=   : 左の部の次数がsup以上のノードを対象とする。省略すればsup=0。
  kn|2 = : kn=で指定された値以上の共起頻度を対象とする。kn2=で指定された値以上の次数を持つ右部を対象とする。
           省略すればkn=1,kn2=1 [1以上の整数]
  iter=  : データ研磨の最大繰り返し数(デフォルト=30)
  log=   : ディレクトリ内にパラメータの設定値や収束回数等をkey-value形式のCSVで出力.繰り返し毎に生成される類似グループ出力


  その他
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

備考)
内部で起動しているコマンドsspcは0から始まる整数で指定された節点名を前提として処理する。
一方で本コマンドは、任意の文字列で節点名を表したデータを処理できる。
それは、sspcを実行する前に、それら文字列と整数との対応表を前処理で作成しているからである。

例) 2部グラフデータからのみグラフ研磨を実行する例
$ cat edge.csv
node1,node2
A,a
A,b
B,a
B,b
C,c
C,d
D,b
D,e

$ #{$cmd} ei=edge.csv ef=n1,n2 th=0.2 eo=output.csv
#MSG# converting the numbered nodes into original name ...; 2015/09/27 21:59:08
#END# #{$cmd} ei=edge.csv ef=n1,n2 th=0.2 eo=output.csv; 2015/09/27 21:59:08
$ cat output.csv
n1,n2
A,a
A,b
A,e
B,a
B,b
B,e
C,c
C,d
D,a
D,b
D,e

# Copyright(c) NYSOL 2012- All Rights Reserved.
		"""

	verInfo="version=1.2"

	paramter = {	
		"ei":"filename",
		"ef":"fld",
		"eo":"filename",		
		"log":"filename",
		"th":"float",
		"th2":"float",
		"sim":"str",
		"sim2":"str",
		"kn":"int",
		"kn2":"int",
		"sup":"int",
		"iter":"int",
		"rp":"bool",
		"T":"str"
	}

	paramcond = {	
		"hissu": ["ei","th"]
	}
	
	def help():
		print(mbipolish.helpMSG) 

	def ver():
		print(mbipolish.verInfo)

	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in mbipolish.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )

		self.msgoff = True

		self.eo      = kwd["eo"]  if "eo"  in kwd else None
		self.logDir  = kwd["log"]  if "log"  in kwd else None
		self.outDir  = kwd["O"]    if "O"    in kwd else None # 過程出力
		self.rpf  = kwd["rp"] if "rp" in kwd else False 

		# ---- edge field names (two nodes) on ei=
		self.ei = kwd["ei"] # edgeファイル名
		ef0 = kwd["ef"].split(",") if "ef"   in kwd else ["node1","node2"]
		self.ef1 = ef0[0]
		self.ef2 = ef0[1] 

		# threashold for similarity measure
		self.th  = float(kwd["th"])  
		self.th2 = float(kwd["th2"]) if "th2" in kwd else self.th  # threashold for similarity measure

		# similarity measure
		self.measure  = kwd["sim"]  if "sim"  in kwd else "R"  
		self.measure2 = kwd["sim2"] if "sim2" in kwd else self.measure 

		self.minSupp  = int(kwd["sup"])  if "sup"  in kwd else 0   # minimam support
		self.iterMax  = int(kwd["iter"]) if "iter" in kwd else 30  # upper bound of iterations
     
		self.kn       = int(kwd["kn"])  if "kn"  in kwd else 1  # no. of interaction size more than threshold 
		self.kn2      = int(kwd["kn2"])  if "kn2"  in kwd else 1         # no. of right node size more than threshold 

		if self.outDir:
			nu.mkDir(outDir,rm=True) 

		if self.logDir:
			nu.mkDir(logDir,rm=True) 

	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 


	def __init__(self,**kwd):

		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)


		#self.__tempW	= mtemp.Mtemp()
		#self.part1 = self.__tempW.file()
		#self.part2 = self.__tempW.file()

	def calGsize(self,file):
		edgesize= nu.mrecount(i=file,nfni=True)


		f=None
		f <<= nm.mcut(i=file,f="0:tra",nfni=True)
		f <<= nm.mtra(f="tra")
		f <<= nm.muniq(k="tra")
		f <<= nm.mcount(a="cnt")
		f <<= nm.mcut(f="cnt")
		f <<= nm.writelist(dtype="cnt:int")
		nodesize = f.run()[0][0]

		return nodesize,edgesize
	
	def same(self,file1,file2,xx):

		if os.path.getsize(file1)!=os.path.getsize(file2):
			return False

		os.system("diff -q %s %s > %s"%(file1,file2,xx))

		if os.path.getsize(xx)!=0:
			return False 
	
		return True




	def edge2mtx(self,ei,itra,map1,map2):
		
		p1   = nm.mcut(f=self.ef1,i=ei) 
		p1 <<= nm.muniq(k=self.ef1)
		p1 <<= nm.mdelnull(f=self.ef1)
		p1 <<= nm.mnumber(q=True,a="num1",S=1,o=map1)

		p2   = nm.mcut(f=self.ef2,i=ei) 
		p2 <<= nm.muniq(k=self.ef2)
		p2 <<= nm.mdelnull(f=self.ef2)
		p2 <<= nm.mnumber(q=True,a="num2",S=1,o=map2)
		
		runp = None
		runp <<= nm.mcut(f=[self.ef1,self.ef2],i=ei)
		runp <<= nm.mjoin(k=self.ef1,m=p1,f="num1")
		runp <<= nm.mjoin(k=self.ef2,m=p2,f="num2")
		runp <<= nm.mcut(f="num1,num2")
		runp <<= nm.mtra(k="num1",f="num2")
		runp <<= nm.msortf(f="num1%n")

		runp <<= nm.mcut(f="num2",nfno=True)
		runp <<= nm.cmd("tr ',' ' '")
		runp <<= nm.mwrite(o=itra)
		#runp <<= nm.mcut(f="num2",nfno=True,o=wff1)
		runp.run()
		#os.system("tr ',' ' ' < {} > {}".format(wff1,itra))

		
	def noPat(self):
		##MCMD::msgLog("There is no frequent item. The value is too large")
		exit()
	
	def convRsl(self,ifile,ofile,map1,map2,logDir=None):

		# 上記iterationで収束したマイクロクラスタグラフを元の節点文字列に直して出力する
		#MCMD::msgLog("converting the numbered nodes into original name ...")
		f = None
		f <<= nm.mcut(nfni=True,f="0:tra",i=ifile)
		f <<= nm.msed(f="tra", c=' $',v="")
		f <<= nm.mnumber(q=True,S=1,a="num1")
		f <<= nm.mtra(r=True,f="tra:num2")
		f <<= nm.mjoin(k="num2",m=map2,f=self.ef2)
		f <<= nm.mjoin(k="num1",m=map1,f=self.ef1)
		f <<= nm.msortf(f="num1%n,num2%n")
		f <<= nm.mcut(f=[self.ef1,self.ef2])

		if logDir:
			f <<= nm.mfldname(q=True , o="{}/#{ofile}".format(logDir,ofile))
		else:
			f <<= nm.mfldname(q=True,o=ofile)

		f.run()

	def convSim(self,ifile,ofile,map1,logDir):

		f = None
		f <<= nm.mcut(nfni=True,f="0:tra",i=ifile)
		f <<= nm.msed(f="tra", c=' $',v="")
		f <<= nm.mnumber(q=True,S=1,a="num1")
		f <<= nm.mtra(r=True,f="tra:num11")
		f <<= nm.mnumber(q=True,S=1,a="order")
		f <<= nm.mcal(c='${num11}+1',a="num1")
		f <<= nm.mjoin(k="num1",m=map1,f=self.ef1)
		f <<= nm.mtra(k="num0",s="order%n,num1%n",f=self.ef1)
		f <<= nm.mcut(f=self.ef1,o="{}/{}".format(logDir,ofile),ro=self.rpf)
		f.run()

	# execute
	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		from datetime import datetime	
		t = datetime.now()

		tempW	= nu.Mtemp()

		xxinp  = tempW.file()
		xxitra = tempW.file()
		xxdiff = tempW.file()
		xxsame = tempW.file()
		xxmap1 = tempW.file()
		xxmap2 = tempW.file()

		self.edge2mtx(self.ei,xxinp,xxmap1,xxmap2)
		extTake.grhfil(type='D""',i=xxinp,o=xxitra)
		
		iter=0
		xxprev = tempW.file()
		xxpair = tempW.file()
		xxtra  = tempW.file()
		
		while True :
			# 終了判定
			if iter>=self.iterMax:
				break
			if iter!=0 and self.same(xxitra,xxprev,xxsame):

				break

			#MCMD::msgLog("polishing iteration ##{iter} (tra size=#{File.size(xxitra)}")
			shutil.copyfile(xxitra,xxprev)
		
			nodeSize,edgeSize=self.calGsize(xxitra)
			edgeSize1 = edgeSize+1

			#print "sspc #{measure} -l #{minSupp} #{xxtra} #{th} #{xxpair}"
			runpara={}
			runpara["type"] = "t"+self.measure 
			runpara["T"] = self.kn
			runpara["l"] = self.minSupp
			runpara["U"] = 100000
			runpara["L"] = 1
			runpara["i"] = xxitra
			runpara["th"] = self.th
			runpara["o"] = xxpair
			extTake.sspc(runpara)

			#	puts   "sspc t#{measure} -T #{kn} -l #{minSupp} -U 100000 -L 1 #{xxitra} #{th} #{xxpair}"

			if not os.path.exists(xxpair):
				self.noPat()
			if os.path.getsize(xxpair) == 0:
				self.noPat()
			
			extTake.grhfil(type='eu0',i=xxpair,o=xxtra)

			if self.logDir :
				self.convSim(xxtra,"simGp{}.csv".format(iter),xxmap1,self.logDir)
			
			os.system("cat {} {} > {}".format(xxitra,xxtra,xxpair))
		
			runpara={}
			runpara["type"] = self.measure2
			runpara["T"] = self.kn2
			runpara["c"] = edgeSize
			runpara["i"] = xxpair
			runpara["th"] = self.th2
			runpara["o"] = xxtra

			extTake.sspc(runpara)

			if not os.path.exists(xxtra):
				self.noPat()
			if os.path.getsize(xxtra) == 0:
				self.noPat()

			extTake.grhfil(type='ed',i=xxtra,o=xxpair)

			os.system("tail -n +{} {} > {}".format(edgeSize1,xxpair,xxtra))

			extTake.grhfil(type='D',i=xxtra,o=xxpair)


			extTake.grhfil(type='DE',d=xxitra,i=xxpair,o=xxdiff) #<==つかってない？



			shutil.copyfile(xxpair,xxitra)

			if self.logDir :
				extTake.grhfil(type='D',i=xxtra,o=xxpair)
				self.convRsl(xxpair,"iter{}.csv".format(iter),xxmap1,xxmap2,logDir)
		
			iter+=1


		extTake.grhfil(type='D',i=xxitra,o=xxpair)
		self.convRsl(xxpair,self.eo,xxmap1,xxmap2)

		procTime = datetime.now()-t

		if self.logDir :

			kv=[["key","value"]]

			for k,v in self.args.items():
				kv.append([k,str(v)])

			kv.append(["iter",str(iter)])
			kv.append(["time",str(procTime)])
			for i in range(len(nSizes)):
				kv.append(["nSize"+str(i),str(nSizes[i])]);
				kv.append(["eSize"+str(i),str(eSizes[i])]);
				kv.append(["dens"+str(i) ,str(denses[i])]);

			nm.writecsv(i=kv,o="{}/keyVal.csv".format(self.logDir)).run()

		nu.mmsg.endLog(self.__cmdline())



# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Mnewnumber(NysolMOD_CORE):

	_kwd ,_inkwd,_outkwd = n_core.getparalist("mnewnumber",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_Mnewnumber,self).__init__("mnewnumber",nutil.args2dict(args,kw_args,Nysol_Mnewnumber._kwd))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs
import nysol.take as nt

args=margs.Margs(sys.argv,"ei=,ef=,o=,l=,u=,-edge,-rp","ei=")
nt.mbiclique(**args.kvmap()).run(msg="on")





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/

from . import _utillib 

def mheader(**kw_args):

	nfnFlg = ("nfni" in kw_args and kw_args["nfni"] == True) or ("nfn" in kw_args and kw_args["nfn"] == True)
	if not "i" in kw_args:
		return None

	return _utillib.mheader(kw_args["i"],nfnFlg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/
import os
import datetime

def errorLog(msg=""):
	vbl = os.getenv("KG_ScpVerboseLevel")
	if not vbl or int(vbl)>=1 :
		t = datetime.datetime.today()
		print("#ERROR# %s; %s"%( msg , t.strftime('%Y/%m/%d %H:%M:%S') ) )

def warningLog(msg=""):
	vbl = os.getenv("KG_ScpVerboseLevel")
	if not vbl or int(vbl) >= 2 :
		t = datetime.datetime.today()
		print("#WARNING# %s; %s"%( msg , t.strftime('%Y/%m/%d %H:%M:%S') ) )

def endLog(msg=""):
	vbl = os.getenv("KG_ScpVerboseLevel")
	if not vbl or int(vbl) >=3 :
		t = datetime.datetime.today()
		print("#END# %s; %s"%( msg , t.strftime('%Y/%m/%d %H:%M:%S') ) )


def msgLog(msg="",time=True,header=True):
	vbl = os.getenv("KG_ScpVerboseLevel")
	if not vbl or int(vbl) >=4 :
		str = ""
		str = "#MSG# " if header else ""
		str = str + msg + "; "

		if time :
			t = datetime.datetime.today()
			str = str + (t.strftime('%Y/%m/%d %H:%M:%S'))
		print(str)

		#STDERR.puts "#END# #{msg}; #{Time.now.strftime('%Y/%m/%d %H:%M:%S')}"
		#STDERR.puts str



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs 
import nysol.take as nt


args=margs.Margs(sys.argv,"i=,c=,x=,O=,tid=,time=,item=,cls=,taxo=,s=,S=,sx=,Sx=,g=,p=,-uniform,l=,u=,top=,gap=,win=,-padding,T=,-mcmdenv,-rp","i=")
nt.msequence(**(args.kvmap())).run(msg="on")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs 
import nysol.take as nt

args=margs.Margs(sys.argv,"i=,no=,eo=,log=,tid=,item=,s=,S=,sim=,th=,-node_support,T=,-num,-rp","i=,tid=,item=,eo=")
nt.mtra2gc(**(args.kvmap())).run(msg="on")





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs
import nysol.take as nt


args=margs.Margs(sys.argv,"ei=,ef=,eI=,eF=,ni=,nf=,nI=,nF=,-dir,eo=,no=,T=,-rp","ei=,eI=,eo=")
nt.mgdiff(**args.kvmap()).run(msg="on")




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import os.path
import shutil

import nysol.mcmd as nm
import nysol.util as nu

#import nysol.util.margs as margs
#import nysol.util.mtemp as mtemp
#import nysol.util.mrecount as mrecount

from nysol.take import extcore as extTake

class mclique(object):

	helpMSG="""
# ver="1.0" # 初期リリース 2014/2/20
# ver="1.1" # eo=,no=の機能追加 2014/3/27
# ver="1.2" # ni=,nf=の追加, eo=,no=の機能をmclique2g.rbに分離, 枝出力を廃止(-nodeによる出力のみ) 2014/8/2
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) 一般グラフデータから極大クリークを列挙する。
書式) #{$cmd} ei= ef= [ni=] [nf=] [o=] [l=] [u=] [-node] [-all] [log=] [T=] [--help]

  ファイル名指定
  ei=    : 枝データファイル
  ef=    : 枝データ上の2つの節点項目名(省略時は"node1,node2")
  ni=    : 節点データファイル
  nf=    : 節点データ上の節点項目名(省略時は"node")
  o=     : 出力ファイル(クリークID-枝:-nodeを指定することでクリークID-節点に変更可能,省略時は標準出力)
  l=     : クリークを構成する最小節点数(ここで指定したサイズより小さいクリークは列挙されない)
  u=     : クリークを構成する最大節点数(ここで指定したサイズより大きいクリークは列挙されない)
  -all   : 極大クリークだけでなく、全クリークを列挙する。
  log=   : パラメータの設定値をkey-value形式のCSVで保存するファイル名

  その他
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

入力形式)
一般グラフを節点ペアで表現した形式。
他のいかなる節点とも接続のない節点は、サイズが１の自明なクリークであるため、入力対象外とする。

o=の出力形式)
クリークIDと接点を出力する。
出力項目は"id,node,size"の3項目である。
sizeはクリークを構成する節点数である。

備考)
内部で起動しているコマンドmaceは0から始まる整数で指定された節点名を前提として処理する。
一方で本コマンドは、任意の文字列で節点名を表したデータを処理できる。
それは、maceを実行する前に、それら文字列と整数との対応表を前処理で作成しているからである。

例)
$ cat data/edge.csv 
n1,n2
a,b
a,c
a,d
b,c
b,d
b,e
c,d
d,e

$ #{$cmd} ei=edge.csv ef=n1,n2 o=output.csv
#MSG# converting graph files into a pair of numbered nodes ...; 2014/01/06 14:27:17
#MSG# converting the numbered nodes into original name ...; 2014/01/06 14:27:17
#END# #{$cmd} ei=data/edge1.csv ef=n1,n2 -node o=output.csv; 2014/01/06 14:27:17
$ cat output.csv 
id,node,size
0,b,3
0,d,3
0,e,3
1,a,4
1,b,4
1,c,4
1,d,4

例) 節点ファイルも指定した例
$ cat edge.csv 
n1,n2
c,a
c,d
d,e
a,d
a,e

$ cat node.csv 
n
a
b
c
d
e
f

$ #{$cmd} ei=edge.csv ef=n1,n2 ni=node.csv nf=n o=output.csv
#MSG# converting graph files into a pair of numbered nodes ...; 2014/01/06 14:27:17
#MSG# converting the numbered nodes into original name ...; 2014/01/06 14:27:17
#END# #{$cmd} ei=data/edge1.csv ef=n1,n2 -node o=output.csv; 2014/01/06 14:27:17
$ cat output.csv 
id,node,size
0,b,1
1,a,3
1,d,3
1,e,3
2,a,3
2,c,3
2,d,3
3,f,1

# Copyright(c) NYSOL 2012- All Rights Reserved.
		"""

	verInfo="version=1.2"

	paramter = {	
		"ei":"filename",
		"ef":"fld",
		"ni":"filename",
		"nf":"fld",
		"l":"int",
		"u":"int",
		"o":"filename",		
		"log":"filename",
		"rp":"bool",
		"all":"bool",
		"T":"str"
	}

	paramcond = {	
		"hissu": ["ei","ef"]
	}

	def help():
		print(mclique.helpMSG) 

	def ver():
		print(mclique.versionInfo)



	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in mclique.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )

		self.msgoff = True

		self.ei = kwd["ei"] # edgeファイル名
		self.ni = kwd["ni"] if "ni"  in kwd else None # node file name
		
		# ---- edge field names (two nodes) on ei=
		ef0 = kwd["ef"].split(",")
		self.ef1 = ef0[0]
		self.ef2 = ef0[1] 

		# ---- node field name on ni=
		self.nf = kwd["nf"] if "nf"  in kwd else "node"

		self.minSize = kwd["l"]   if "l"   in kwd else None   # クリークサイズ下限
		self.maxSize = kwd["u"]   if "u"   in kwd else None   # クリークサイズ上限
		self.oFile   = kwd["o"]   if "o"   in kwd else None
		self.logFile = kwd["log"] if "log" in kwd else None
		self.rpf  = kwd["rp"] if "rp" in kwd else False 

		self.all = kwd["all"] if "all" in kwd else False

	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 

	def __init__(self,**kwd):

		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)


	def g2pair(self,ni,nf,ei,ef1,ef2,ipair,mapFile):

		dlist =[nm.mcut(f=ef1+":node",i=ei),nm.mcut(f=ef2+":node",i=ei)]

		if(ni): 
			dlist.append(nm.mcut(f=nf+":node",i=ni))
			
		fmap = None	
		fmap <<= nm.mcut(i=dlist,f="node")
		fmap <<= nm.muniq(k="node")
		fmap <<= nm.mnumber(q=True,a="num",o=mapFile)

		f =   nm.mcut(f=[ef1,ef2],i=ei)
		f <<= nm.mjoin(k=ef1,K="node",m=fmap,f="num:num1")
		f <<= nm.mjoin(k=ef2,K="node",m=fmap,f="num:num2")
		f <<= nm.mcut(f="num1,num2")
		f <<= nm.mfsort(f="num1,num2")
		f <<= nm.msortf(f="num1%n,num2%n" , nfno=True,o=ipair)
		f.run()
	# ============
	# entry point
	def run(self,**kw_args):
	
		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		from datetime import datetime	
		t = datetime.now()

		wf = nu.Mtemp()
		xxinp    = wf.file()
		xxmap    = wf.file()
		input    = self.ei

		self.g2pair(self.ni,self.nf,self.ei,self.ef1,self.ef2,xxinp,xxmap)

		xxmace = wf.file() # maceの出力(tra形式)

		para = {}
		if self.msgoff :
			para["type"] = "Ce_" if self.all else "Me_"
		else:
			para["type"] = "Ce" if self.all else "Me"
		para["i"] = xxinp
		para["o"] = xxmace
		if self.minSize :
			para["l"] = self.minSize
		if self.maxSize :
			para["u"] = self.maxSize
		extTake.mace(para)
		
		#MCMD::msgLog("converting the numbered nodes into original name ...")
		id = nu.mrecount(i=xxmace,nfni=True)

		# xxpair = wf.file() # 上記traをpair形式に変換したデータ

		fpair = None
		fpair <<= nm.mcut(i=xxmace,nfni=True,f="0:num")
		fpair <<= nm.mnumber(q=True,a="id")
		fpair <<= nm.mvcount(vf="num:size")
		fpair <<= nm.mtra(r=True,f="num")
		

		# when ni= specified, it add the isolated single cliques.
		if self.ni :

			fpair_u = nm.mread(i=fpair)

			if self.all:
				fpair_u <<= nm.mselstr(f="size",v=1)
			fpair_u <<= nm.mcut(f="num")
			fpair_u <<= nm.muniq(k="num")

			# select all nodes which are not included in any cliques
			xxiso=None
			xxiso <<= nm.mcut(f="num",i=xxmap)
			xxiso <<= nm.mcommon(k="num",m=fpair_u,r=True)
			xxiso <<= nm.mnumber(S=id,a="id",q=True)
			xxiso <<= nm.msetstr(v=1,a="size")
			xxiso <<= nm.mcut(f="id,num,size")
			# mcut入れないとおかしくなるあとで直す
			#ddlist = [fpair.mcut(f="id,num,size"),xxiso]
			xxpair = nm.mcut(i=[fpair,xxiso],f="id,num,size")
			
		else:

			xxpair = fpair


		xxpair <<= nm.mjoin(m=xxmap,k="num",f="node")
		xxpair <<= nm.mcut(f="id,node,size")
		xxpair <<= nm.msortf(f="id,node",o=self.oFile,rp=self.rpf)
		xxpair.run()

		procTime = datetime.now()-t

		# ログファイル出力
		if self.logFile :
			kv=[["key","value"]]
			for k,v in self.args.items():
				kv.append([k,str(v)])
			kv.append(["time",str(procTime)])
			nm.writecsv(i=kv,o=self.logFile).run()

		nu.mmsg.endLog(self.__cmdline())






#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs
import nysol.take as nt


args=margs.Margs(sys.argv,"ei=,ef=,ni=,nf=,o=,-verbose,-rp","ei=,ef=")
nt.mccomp(**(args.kvmap())).run(msg="on")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/
import os
import shutil
import nysol.util as nu
import nysol.mcmd as nm
from nysol.take import extcore as extTake


#========================================================================
# 列挙関数:lcm_seq 利用DB:SeqDB
#========================================================================
class LcmEsp(object):

	intMax=2147483646

	# posトランザクションの重み計算
	# マニュアルの式(10)
	def calOmega(self,posCnt):
		return LcmEsp.intMax/posCnt


	# LCM最小サポートの計算
	# マニュアルの式(9)
	def calSigma(self,minPos,minGR,posCnt,negCnt):
		omegaF=self.calOmega(posCnt)
		beta=minPos
		w=float(negCnt)/float(posCnt)
		sigma=int(beta*(omegaF-w/minGR))  # 切り捨て
		if sigma<=0:
			sigma=1 
		return sigma


	# 1:β   最小支持度
	# 2:γ   最小GR
	# 3:Dp  クラス1トランザクション数
	# 4:Dn  クラス2トランザクション数
	# 5:lcmq   LCM列挙数調整パラメータ
  # 1:γ 2:Dp 3:Dn 4:lcmq 
  # 未使用
	#def calOmega(minGR,posCnt,negCnt,lcmq=0.5)
	#	#return (negCnt.to_f*(1-prob))/(posCnt.to_f*prob)/(1-lcmq)
	#	return negCnt.to_f/posCnt.to_f/minGR.to_f/(1-lcmq)
	#end
	#def calAlphaD(minSup,minGR,posCnt,negCnt,lcmq=0.5)
	#	return lcmq*minSup.to_f*negCnt.to_f/minGR.to_f/(1-lcmq)
	#end


	def __init__(self,db,outtf=True):
		self.size  = None
		self.msgoff = True

		self.temp  = nu.Mtemp()
		self.db    = db # 入力データベース
		self.file  = self.temp.file()
		self.outtf = outtf
		items      = self.db.items

		# 重みファイルの作成
		# pos,negのTransactionオブジェクトに対してLCMが扱う整数アイテムによるトランザクションファイルを生成する。
		# この時、pos,negを併合して一つのファイルとして作成され(@wNumTraFile)、
		# 重みファイル(@weightFile[クラス])の作成は以下の通り。
		# 1.対象クラスをpos、その他のクラスをnegとする。
		# 2. negの重みは-1に設定し、posの重みはcalOmegaで計算した値。
		# 3.@wNumTraFileの各行のクラスに対応した重みデータを出力する(１項目のみのデータ)。
		self.weightFile = {}
		self.posWeight  = {}
		self.sigma      = {}

		for cName,posSize in db.clsNameRecSize.items(): 
			self.weightFile[cName] = self.temp.file()
			self.posWeight[cName]  = self.calOmega(posSize)
			cpara = "%s:%s"%(cName,self.posWeight[cName])
			nm.mcut(nfno=True,f=self.db.clsFN,i=self.db.cFile).mchgstr(nfn=True,f=0,O=-1,o=self.weightFile[cName],c=cpara).run()


		# アイテムをシンボルから番号に変換する。
		f=None
		f <<= nm.mjoin(k=self.db.itemFN,K=items.itemFN,m=items.file,f=items.idFN,i=self.db.file)
		f <<= nm.mcut(f=self.db.idFN+","+self.db.timeFN+","+items.idFN)
		f <<= nm.msortf(f=self.db.idFN+","+self.db.timeFN+"%n")
		f <<= nm.mtra(k=self.db.idFN,f=items.idFN)
		f <<= nm.mcut(f=items.idFN,nfno=True,o=self.file)
		f.run()

	def enumerate(self,eArgs):
		tf=nu.Mtemp()

		# 最小サポートと最小サポート件数
		if "minCnt" in eArgs :
			self.minCnt = int(eArgs["minCnt"])
			self.minSup = float(self.minCnt)/ float(self.db.size)
		else:
			self.minSup = float(eArgs["minSup"])
			self.minCnt = int(self.minSup * float(self.db.size) + 0.99)


		# 最大サポートと最大サポート件数 # いらない　<= 下で再セット
		self.maxCnt=None 
		if "maxCnt" in eArgs or "maxSup" in eArgs:
			if "maxCnt" in eArgs:
				self.maxCnt = int(eArgs["maxCnt"])
				self.maxSup = float(self.maxCnt)/float(self.db.size)
			else:
				self.maxSup = float(eArgs["maxSup"])
				self.maxCnt = int(self.maxSup * float(self.db.size) + 0.99)

		#未使用
		#@minProb = eArgs["minProb"].to_f # 事後確率
		#@minGR   = @minProb/(1-@minProb) # 増加率
		#@minGR   = eArgs["minGR"].to_f if eArgs["minGR"]

		# あるクラスをpos、他のクラスをnegにして、パターン列挙した結果ファイル名を格納する
		pFiles=[]
		tFiles=[]
		for cName,posSize in self.db.clsNameRecSize.items(): 
			negSize=self.db.size-posSize

			# minGRの計算
			if "minGR" in eArgs:
				self.minGR=eArgs["minGR"]
			else:
				minProb = eArgs["minProb"] if ( "minProb" in eArgs ) else 0.5
				if "uniform" in eArgs and eArgs["uniform"]:
					self.minGR = (minProb/(1-minProb)) * (self.db.clsSize-1) # マニュアルの式(4)
				else:
					self.minGR = (minProb/(1-minProb)) * (float(negSize)/float(posSize)) # マニュアルの式(4)


			# 最小サポートと最小サポート件数
			if "minCnt" in eArgs:
				self.minPos = eArgs["minCnt"]
			else:
				self.minPos = int(eArgs["minSup"] * float(posSize) + 0.99)

			# 最大サポートと最大サポート件数
			if "maxCnt" in eArgs or "maxSup" in eArgs:
				if "maxCnt" in eArgs:
					self.maxCnt = int(eArgs["maxCnt"])
				else:
 					self.maxCnt = int(eArgs["maxSup"] * float(posSize) + 0.99)


			self.sigma[cName] = self.calSigma(self.minPos,self.minGR,posSize,negSize)

			# lcm_seqのパラメータ設定と実行
			lcmout = tf.file() # lcm_seq出力ファイル
			# 頻出パターンがなかった場合、lcm出力ファイルが生成されないので
			# そのときのために空ファイルを生成しておいく。
			with open(lcmout, "w") as efile:
				pass

			params = {}
			if self.msgoff:
				params["type"] ="CIA_"
			else:
				params["type"] ="CIA"

			if self.maxCnt: # windowサイズ上限
				#params["U"] = str(self.maxCnt)
				params["U"] = self.calSigma(self.maxCnt,self.minGR,posSize,negSize)

			if "minLen" in eArgs:
				params["l"] = str(eArgs["minLen"])

			if 'maxLen' in eArgs:
				params["u"] = str(eArgs["maxLen"])

			if 'gap' in eArgs:
				params["g"] = str(eArgs["gap"])

			if 'win' in eArgs:
				params["G"] = str(eArgs["win"])

			params["w"] = self.weightFile[cName]
			params["i"] = self.file
			params["sup"] = str(self.sigma[cName])
			params["o"] = lcmout

			# lcm_seq実行
			#MCMD::msgLog("#{run}")
			if 'padding' in eArgs and eArgs["padding"]: # padding指定時は、0アイテムを出力しないlcm_seqを実行
				extTake.lcmseq_zero(params)
			else:
				extTake.lcmseq(params)

			# パターンのサポートを計算しCSV出力する
			#MCMD::msgLog("output patterns to CSV file ...")
			pFiles.append(self.temp.file())
			transle = self.temp.file()

			extTake.lcmtrans(lcmout,"e",transle) # pattern,countP,countN,size,pid

			f=None
			f <<= nm.mdelnull(f="pattern",i=transle)
			f <<= nm.mcal(c='round(${countN},1)',a="neg")
			f <<= nm.mcal(c='round(${countP}/%s,1)'%(self.posWeight[cName]),a="pos")
			f <<= nm.mdelnull(f="pattern")
			f <<= nm.msetstr(v=cName,a="class")
			f <<= nm.msetstr(v=posSize,a="posTotal")
			f <<= nm.msetstr(v=self.minGR,a="minGR")
			f <<= nm.mcut(f="class,pid,pattern,size,pos,neg,posTotal,minGR",o=pFiles[-1])
			f.run()

			#s = MCMD::mrecount("i=#{pFiles.last}") # 列挙されたパターンの数
			#MCMD::msgLog("the number of contrast patterns on class `#{cName}' enumerated is #{s}")

			if self.outtf :
				# トランザクション毎に出現するシーケンスを書き出す
				#MCMD::msgLog("output tid-patterns ...")
				tFiles.append(self.temp.file())

				xxw= tf.file()
				f=None
				f <<= nm.mcut(f=self.db.idFN,i=self.db.file)
				f <<= nm.muniq(k=self.db.idFN)
				f <<= nm.mnumber(S=0,a="__tid",q=True)
				f <<= nm.msortf(f="__tid",o=xxw)
				f.run()

				nm.mcut(f=self.db.idFN,i=self.db.file).muniq(k=self.db.idFN).mnumber(S=0,a="__tid",q=True,o=xxw).run()
				translt = self.temp.file()
				extTake.lcmtrans(lcmout,"t",translt)
				nm.mjoin(k="__tid",m=xxw,f=self.db.idFN,i=translt).msetstr(v=cName,a="class").mcut(f=self.db.idFN+",class,pid",o=tFiles[-1]).run()


		# クラス別のパターンとtid-pidファイルを統合して最終出力
		self.pFile = self.temp.file()
		self.tFile = self.temp.file()

		# パターンファイル併合
		xxpCat = tf.file()
		f =   nm.mcat(i=",".join(pFiles))
		f <<= nm.msortf(f="class,pid")
		f <<= nm.mnumber(s="class,pid",S=0,a="ppid",o=xxpCat)
		f.run()

		# パターンファイル計算
		items=self.db.items
		f=""
		f =   nm.mcut(f="class,ppid:pid,pattern,size,pos,neg,posTotal,minGR",i=xxpCat)
		f <<= nm.msetstr(v=self.db.size,a="total")
		f <<= nm.mcal(c='${total}-${posTotal}',a="negTotal") # negのトータル件数
		f <<= nm.mcal(c='${pos}/${posTotal}',a="support") # サポートの計算
		f <<= nm.mcal(c='if(${neg}==0,1.797693135e+308,(${pos}/${posTotal})/(${neg}/${negTotal}))',a="growthRate")
		if "uniform" in eArgs and eArgs["uniform"] == True:
			f <<= nm.mcal(c='(${pos}/${posTotal})/(${pos}/${posTotal}+(%s-1)*${neg}/${negTotal})'%(self.db.clsSize),a="postProb")
		else:
			f <<= nm.mcal(c='${pos}/(${pos}+${neg})',a="postProb")

		f <<= nm.msel(c='${pos}>=%s&&${growthRate}>=${minGR}'%(self.minPos)) # minSupとminGRによる選択
		f <<= nm.mvreplace(vf="pattern",m=items.file,K=items.idFN,f=items.itemFN)
		f <<= nm.mcut(f="class,pid,pattern,size,pos,neg,posTotal,negTotal,total,support,growthRate,postProb")
		f <<= nm.mvsort(vf="pattern")
		f <<= nm.msortf(f="class%nr,postProb%nr,pos%nr",o=self.pFile)
		f.run()

		if self.outtf :
			# 列挙されたパターンを含むtraのみ選択するためのマスタ
			xxp4=nm.mcut(f="class,pid",i=self.pFile)

			f =   nm.mcat(i=",".join(tFiles))
			f <<= nm.mjoin(k="class,pid",m=xxpCat,f="ppid") # 全クラス統一pid(ppid)結合
			f <<= nm.mcommon(k="class,ppid",K="class,pid",m=xxp4) # 列挙されたパターンの選択
			f <<= nm.mcut(f=self.db.idFN+",class,ppid:pid")
			f <<= nm.msortf(f=self.db.idFN+",class,pid",o=self.tFile)
			f.run()


		self.size = nu.mrecount(i=self.pFile)
		#MCMD::msgLog("the number of emerging sequence patterns enumerated is #{@size}")

	def output(self,outpath):
		nm.mfldname(q=True,i=self.pFile,o=outpath+"/patterns.csv").run()
		#shutil.move(self.pFile,outpath+"/patterns.csv")
		if self.outtf:
			nm.mfldname(q=True,i=self.tFile,o=outpath+"/tid_pats.csv").run()
			#shutil.move(self.tFile,outpath+"/tid_pats.csv")





from .margs import Margs
from .mheader import mheader
from .mmkdir import mkDir
from .mmsg import errorLog,warningLog,endLog,msgLog
from .mrecount import mrecount
from .mtemp import Mtemp
from .mchkcsv import mchkcsv
from .margv2dict import margv2dict



# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_List2Csv(NysolMOD_CORE):

	_kwd = [["i","o","header"],[]]
	_inkwd  = ["i"] 
	_outkwd = ["o"]

	def __init__(self,*args, **kw_args) :

		if len(args) != 1 :
			print("arge only one")
			return None

		if isinstance(args[0],list) :
			kw_args["i"] = args[0]
		else :
			print("unsuport type")
			return None
			
		if not isinstance(kw_args["i"],list) :
			print("unsuport type")
			return None

		# headerがあればmcut 		
		if "header" in kw_args :

			if isinstance(kw_args["header"],list) :
				fld =[]
				for i,v in enumerate(kw_args["header"]):
					fld.append("%d:%s"%(i,v))
				
				kw_args["f"] = fld
	
			elif isinstance(kw_args["header"],str) :
				hd = re.split(r",", kw_args["header"])
				for i,v in enumerate(hd):
					fld.append("%d:%s"%(i,v))
			else :
				print("unsuport type")
				return None

			kw_args["nfni"] = True
			del kw_args["header"]
			super(Nysol_List2Csv,self).__init__("mcut",kw_args)

		else:
			kw_args["f"] = "*"		
			super(Nysol_List2Csv,self).__init__("mcut",kw_args)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import shutil
import os
import nysol.util.mtemp as mtemp
import nysol.util.mrecount as mrecount

import nysol.mcmd as nm

from nysol.take.lib import items as items


#=トランザクションデータクラス
# 頻出パターンマイニングで使われるトランザクションデータを扱うクラス。
# トランザクションファイルは、トランザクションID項目とアイテム集合項目から構成される。
# アイテム集合は、スペースで区切られた文字列集合として表現される。
#
#===利用例
# 以下tra.csvの内容
# ---
# items
# a b d
# a d e
# c e
# a d
# c d e
# c e
# a d e
#
# 以下taxo.csvの内容
# ---
# item,taxo
# a,X1
# b,X1
# c,X2
# d,X2
# e,X2
#
# 以下rubyスクリプト
# ---
# require 'rubygems'
# require 'mining'
#
# tra=SeqDB.new("tra.csv",nil,"items")
# puts tra.file       # => ./1252813069_2179/dat/1
# puts tra.numTraFile # => nil
# puts tra.idFN       # => tid
# puts tra.itemsetFN  # => items
# puts tra.recCnt     # => 7
# p    tra.items      # => #<Items:0x16a39ec @iItemFN=["items"],...
#
# taxo=Taxonomy.new("taxo.csv","item","taxo")
# tra.addTaxo(taxo)
# tra.mkNumTra
# puts tra.numTraFile #=> ./1252813069_2179/dat/6
# p    tra.items      #=> #<Items:0x16a39ec @iItemFN=["items", "taxo"],..., @taxonomy=#<Taxonomy:0x168ba7c ...
#
#=====./1252813069_2179/dat/1 の内容
# tid,items
# 1,a b d
# 2,a d e
# 3,c e
# 4,a d
# 5,c d e
# 6,c e
# 7,a d e
#
#=====./1252813069_2179/dat/6 の内容
# 1 2 3 4 5 6 7
# 1 2 4 6 7
# 1 4 5 6 7
# 3 5 7
# 1 4 6 7
# 3 4 5 7
# 3 5 7
# 1 4 5 6 7
#
class SeqDB(object):

	#=== SeqDBクラスの初期化
	#
	#====引数
	# iFile: transactionファイル名
	# iItemsetFN: iFile上のアイテム集合項目名
	# idFN: 新しく命名するトランザクションID項目名
	# itemsetFN: 新しく命名するアイテム集合項目名
	#
	#====機能
	#* トランザクションIDとアイテム集合の2項目から構成するトランザクションデータを新たに作成する。
	#* ID項目が指定されていなければ、1から始まる連番によってID項目を新規に作成する。
	#* トランザクション件数(iFileのレコード件数)を計算する。
	#* アイテム集合項目からItemsオブジェクトを生成する。

	def __init__(self,iFile,idFN,timeFN,itemFN,padding,clsFN=None):
		self.file = None           # トランザクションファイル名
		self.idFN = None          # トランザクションID項目名(String)
		self.timeFN = None        # 時間項目名(String)
		self.itemFN = None        # アイテム集合項目名(String)
		self.clsFN = None         # クラス項目名(String)
		self.size = None          # トランザクションサイズ(Num)
		self.items = None         # Itemsクラス
		self.taxonomy = None      # 階層分類クラス
		self.clsNameRecSize = None # クラス別件数
		self.clsSize = None       # クラス数
		self.cFile = None         # クラスファイル

		self.temp=mtemp.Mtemp()
		self.iFile  = iFile                       # 入力ファイル
		self.iPath  = os.path.abspath(self.iFile) # フルパス
		self.idFN   = idFN                     # トランザクションID項目名
		self.timeFN = timeFN                   # 時間項目名
		self.itemFN = itemFN                   # アイテム項目名
		self.file   = self.temp.file()               # 出力ファイル名
		self.padding= padding                  # 時系列パディング

		# padding指定の場合は、asciiコードの先頭文字列を挿入
		f=None
		if self.padding :
			f <<= nm.mcut(f=self.idFN+","+self.timeFN+","+self.itemFN,i=self.iFile)
			f <<= nm.msortf(f=self.idFN+","+self.timeFN+"%n,"+self.itemFN)
			f <<= nm.muniq(k=self.idFN+","+self.timeFN+","+self.itemFN)
			f <<= nm.mpadding(k=self.idFN,f=self.timeFN+"%n",v="!",o=self.file)
		else:
			f <<= nm.mcut(f=self.idFN+","+self.timeFN+","+self.itemFN,i=self.iFile)
			f <<= nm.msortf(f=self.idFN+","+self.timeFN+"%n,"+self.itemFN)
			f <<= nm.muniq(k=self.idFN+","+self.timeFN+","+self.itemFN,o=self.file)

		f.run()

		# レコード数の計算
		#@recCnt = MCMD::mrecount("i=#{@file}")

		# トランザクション数の計算
		xx1 = nm.mcut(f=self.idFN,i=self.file).muniq(k=self.idFN).mcount(a="__cnt").mcut(f='__cnt').run()
		self.size = int(xx1[0][0])


		# トランザクションデータからアイテムオブジェクトを生成
		if self.padding : #同じ？
			# paddingの場合は、特殊アイテム(!)を0番とし、出力を制御する。
			self.items=items.Items(self.file,self.itemFN)
		else:
			self.items=items.Items(self.file,self.itemFN)

		if clsFN :
			self.clsFN=clsFN
			self.cFile=self.temp.file()
			fpara_c = "%s,%s"%(self.idFN,self.clsFN)
			nm.mcut(f=fpara_c,i=self.iFile).muniq(k=fpara_c,o=self.cFile).run()

			# 文字列としてのクラス別件数配列を数値配列に変換する
			self.clsSize=0
			self.clsNames = []
			self.clsNameRecSize = {}
			for vv in nm.mcut(f=self.clsFN,i=self.cFile).mcount(k=self.clsFN,a='count'):
				self.clsNames.append(vv[0])	 
				self.clsNameRecSize[vv[0]] = int(vv[1]) 
				self.clsSize+=1

	def show(self):
		print("#### BEGIN SeqDB class")
		print("@temp="+self.temp)
		print("@iFile="+self.iFile)
		print("@iPath="+self.iPath)
		print("@idFN="+self.idFN)
		print("@timeFN="+self.timeFN)
		print("@itemFN="+self.itemFN)
		print("@padding="+self.padding)
		print("@file="+self.file)
		self.items.show()
		os.system("head "+self.file)
		print("#### END SeqDB class")


	def replaceFile(self,train):

		self.file   = train

		# レコード数の計算
		#self.recCnt = nutil.mrecount(i=self.file)

		xx1 = nm.mcut(f=self.idFN,i=self.file).muniq(k=self.idFN).mcount(a='__cnt').mcut(a='__cnt').run()
		self.size = int(xx1[0][0])


	#=== taxonomyをトランザクションに追加
	# トランザクションデータのアイテム集合に、対応するtaxonomyを追加する。
	#
	#====引数
	# taxonomy: Taxonomyオブジェクト。
	#
	#====機能
	#* トランザクションデータのアイテム集合項目におけるアイテム全てについて、対応するtaxonomyをアイテム集合として追加する。
	def repTaxo(self,taxonomy):

		#@taxonomy=taxonomy

		self.items.repTaxo(taxonomy) # アイテムクラスをtaxonomyで置換する

		tFile =taxonomy.file
		itemFN=taxonomy.itemFN
		taxoFN=taxonomy.taxoFN

		tf=mtemp.Mtemp()
		xx1=tf.file()
		f=None
		f <<= nm.mjoin(k=self.itemFN,K=itemFN,f=taxoFN,m=tFile,i=self.file)
		f <<= nm.mcut(f=self.idFN+","+self.timeFN+","+taxoFN+":"+self.itemFN)
		f <<= nm.msortf(f=self.idFN+","+self.timeFN+","+self.itemFN ,o=xx1)
		f.run()

		self.file=self.temp.file()
		shutil.move(xx1,self.file)




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Mmvavg(NysolMOD_CORE):

	_kwd,_inkwd,_outkwd  = n_core.getparalist("mmvavg",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_Mmvavg,self).__init__("mmvavg",nutil.args2dict(args,kw_args,Nysol_Mmvavg._kwd))


def mmvavg(self,*args, **kw_args):
	return Nysol_Mmvavg(nutil.args2dict(args,kw_args,Nysol_Mmvavg._kwd)).addPre(self)

setattr(NysolMOD_CORE, "mmvavg", mmvavg)



from . import _grhfillib 
from . import _lcmlib 
from . import _lcmseq_zerolib 
from . import _lcmseqlib  
from . import _lcmtranslib 
from . import _mmaketralib 
from . import _macelib 
from . import _medsetlib 
from . import _simsetlib 
from . import _sspclib 
from . import extcore

from .mitemset import mitemset as mitemset 
from .msequence import msequence as msequence
from .mbiclique import mbiclique as mbiclique
from .mbipolish import mbipolish as mbipolish
from .mclique import mclique as mclique
from .mccomp import mccomp as mccomp
from .mfriends import mfriends as mfriends
from .mgdiff import mgdiff as mgdiff
from .mpal import mpal as mpal
from .mpolishing import mpolishing as mpolishing
from .mtra2gc import mtra2gc as mtra2gc



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import os.path
import shutil

import nysol.mcmd as nm
import nysol.util as nu

import nysol.util.mtemp as mtemp
from nysol.take import graph as ntg
from nysol.take import extcore as extTake


class mpolishing(object):

	helpMSG="""
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) グラフデータの研磨をおこなう。
内容) 一般グラフデータを入力として、密度の高い部分グラフにあって枝が張られていないノードペアに枝を張る。
      逆に、密度の低い部分グラフにあって枝が張られているノードペアの枝を刈る。
      新たに張られる枝や刈られる枝の程度は、sim=とth=で与えた値によって変わる。
      詳細は以下に示したアルゴリズムを参照。
書式) #{$cmd} ei= [ef=] [ni=] [nf=] eo= [no=] [sim=i|I|s|S|T|R|P|C] th= [sup=] [-indirect] [iter=] [log=] [T=] [--help]

  ファイル名指定
  ei=    : 枝データファイル
  ef=    : 枝データ上の2つの節点項目名(省略時は"node1,node2")
  ni=    : 節点データファイル
  nf=    : 節点データ上の節点項目名(省略時は"node")
  eo=    : データ研磨後の枝データファイル
  no=    : データ研磨後の節点データファイル
  sim=   : 節点a,bと接続された枝集合を、それぞれA,Bとすると、節点a,bに枝を張るために用いる類似度。
           省略時はRが設定される。
             i: inclusion
             I: both-inclusion
             S: |A∩B|/max(|A|,|B|)
             s: |A∩B|/min(|A|,|B|)
             T (intersection): find pairs having common [threshld] items
             R (resemblance): find pairs s.t. |A\capB|/|A\cupB| >= [threshld]
             P (PMI): find pairs s.t. log (|A\capB|*|all| / (|A|*|B|)) >= [threshld]
             C (cosine distance): find pairs s.t. inner product of their normalized vectors >= [threshld]
  th=    : sim=で指定された類似度について、ここで指定された値以上の節点間に枝を張る。
  sup=   : 類似度計算において、|A∩B|>=supの条件を加える。省略すればsup=0。
  -indirect: 上記類似度計算における隣接節点集合から直接の関係を除外する。
             すなわち、A=A-b, B=B-a として類似度を計算する。
  iter=  : データ研磨の最大繰り返し数(デフォルト=30)
  log=   : パラメータの設定値や収束回数等をkey-value形式のCSVで保存するファイル名

  その他
  T= : ワークディレクトリ(default:/tmp)
  O=     : デバッグモード時、データ研磨過程のグラフを保存するディレクトリ名
  --help : ヘルプの表示

備考)
内部で起動しているコマンドsspcは0から始まる整数で指定された節点名を前提として処理する。
一方で本コマンドは、任意の文字列で節点名を表したデータを処理できる。
それは、sspcを実行する前に、それら文字列と整数との対応表を前処理で作成しているからである。

アルゴリズム)
  # 本コマンドでデフォルトで設定されている類似度関数。
  function sim(E,a,b,th,minSup)
    A=E上での節点aの隣接節点集合
    B=E上での節点bの隣接節点集合
		if |A ∩ B|>=minSup and |A ∩ B|/|A ∪ B|>=th
			return TRUE
		else
			return FALSE
		end
  end

  function conv(E,sim,th,minSup)
    foreach(e∈E)
      a,b=eを構成する2つの節点
      if sim(E,a,b,th,minSup)
        E' = E' ∪ edge(a,b)
      end
    end
    return E'
  end

  polishing(E,sim,th,minSup,iter)
    E:=グラフを構成する枝集合
    iter:=最大繰り返し回数
    sim:=類似度関数
    th:=最小類似度
    c=0
    while(c<iter)
      E'=conv(E,sim,th,minSup)
      if E==E'
        break
      end
      E=E'
      c=c+1
    end
    return E
  end

例) 枝データからのみグラフ研磨を実行する例
$ cat edge.csv
n1,n2
a,b
a,d
b,c
b,d
c,d
d,e
$ #{$cmd} ei=edge.csv ef=n1,n2 th=0.5 eo=output.csv
#MSG# converting graph files into a pair of numbered nodes ...; 2013/10/10 14:48:03
#MSG# polishing iteration #0; 2013/10/10 14:48:04
#MSG# converting the numbered nodes into original name ...; 2013/10/10 14:48:04
#END# #{$cmd} ei=edge.csv ef=n1,n2 th=0.1 eo=output.csv; 2013/10/10 14:48:04
$ cat output.csv
n1,n2
a,b
a,c
a,d
b,c
b,d
c,d

# Copyright(c) NYSOL 2018- All Rights Reserved.
		"""

	verInfo="version=1.2"

	def help():
		print(mpolishing.helpMSG) 

	def ver():
		print(mpolishing.versionInfo)

	def __init__(self,gi=None,go=None,sim="R",th=None,indirect=False,sup=0,iter=30,rp=None,O=None,log=None):

		self.gi				= None			# (graph object) input graph set
		self.go				= None			# (graph object) output graph set
		self.__eo			= None			# (string) output edge file name
		self.__no			= None			# (string) output node file name
		self.__nf			= None			# (string) output node file column title
		self.th				= float(th)	# (float) threshhold of degree of similarity
		self.indirect	= indirect	# (boolean) exclude direct relationship from adjacent node set in similarity calculation
		self.measure	= sim				# (string) similarity measure
		self.minSupp	= int(sup) # (float) minimum support
		self.iterMax	= int(iter)	    # (int) upper bound of iterations
		self.logFile	= log				# (string) log file name
		self.outDir		= O					# (string) directory name for outputs in the process
		self.msgoff		= True
		self.__tempW	= mtemp.Mtemp()	# temp files created by this instance will be deleted, after self.run() executed.

		if(gi):
			self.gi = gi						# input graph object
		else:
			raise Exception("gi is mandatory.")
		
		if(go):
			self.go = go
		self.__eo = self.__tempW.file() 
		self.__no = self.__tempW.file()
		self.__nf = "n"
		
		if self.iterMax < 1:
			raise Exception("iter must be 1 or more.")

		if self.outDir and not os.path.isdir(self.outDir) :
			os.makedirs(self.outDir)
			
		self.args = {}
		self.args["sim"] = sim 
		self.args["th"] = th 
		self.args["indirect"] = indirect 
		self.args["sup"] = sup
		self.args["iter"] = iter
		if O != None :
			self.args["O"] = O 
		if log != None :
			self.args["log"] = log 

	# count node size and edge size for sspc's output file
	def calGsize(self,file):
		nodes = set([])
		edgeSize=0
		with open(file) as f:
			for line in f:
				line = line.rstrip('\r\n')
				n1,n2=line.split(" ")
				nodes.add(n1)
				nodes.add(n2)
				edgeSize+=1

		return len(nodes),edgeSize

	# calculate graph features of sspc's output file
	def features(self,file,orgNsize=None):
		nodeSize,edgeSize=self.calGsize(file)
		nSize=float(nodeSize)
		if(orgNsize):
			nSize=float(orgNsize)
		dens=None
		if(nSize>1):
			dens=float(edgeSize)/(nSize*(nSize-1)/2.0) 
		return nSize,edgeSize,dens

	# compare sspc' output files before and after
	def same(self,file1,file2):
		xx = self.__tempW.file()
		if os.path.getsize(file1)!= os.path.getsize(file2):
			return False
		os.system("diff -q %s %s > %s"%(file1,file2,xx))
		if(os.path.getsize(xx) != 0):
			return False
		return True


	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 


	# execute
	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		from datetime import datetime	
		t = datetime.now()

		input=self.gi.eFileT
		xxmap = self.gi.mFile
		xxmaprev = self.__tempW.file()
		nm.msortf(f="id",i=xxmap,o=xxmaprev).run()


		xxpair = self.__tempW.file() # sscpの出力(pair形式)
		xxtra  = self.__tempW.file() # sscpの入力(tra形式)
		xxprev = self.__tempW.file() # 前回のxxtra
		xxtmmp = self.__tempW.file()

		shutil.copyfile(input,xxpair)

		nSizes=[]
		eSizes=[]
		denses=[]

		iter=0
		while True :
			# グラフ特徴量の計算
			if(self.logFile):
				nSize,eSize,dens=self.features(xxpair)
				nSizes.append(nSize)
				eSizes.append(eSize)
				denses.append(dens)

			# node pairをsspc入力形式に変換

			if self.indirect :
				gtpstri = "ue_" if self.msgoff else "ue"
				extTake.grhfil(type=gtpstri,i=xxpair,o=xxtra)
			else:
				gtpstri0 = "ue0_" if self.msgoff else "ue0"
				extTake.grhfil(type=gtpstri0,i=xxpair,o=xxtra)
				
			para = "%s,%s"%(self.gi.edgeFN1,self.gi.edgeFN2)

			if self.outDir :
				os.system("tr ' ' ',' < %s > %s "%(xxpair,xxtmmp))
				f = nm.mcut(f="0:num1,1:num2",nfni=True,i=xxtmmp)
				f <<= nm.mjoin(k="num1",K="id",m=xxmaprev,f="node:%s"%(self.gi.edgeFN1))
				f <<= nm.mjoin(k="num2",K="id",m=xxmaprev,f="node:%s"%(self.gi.edgeFN2)).mcut(f=para).mfsort(f=para)
				f <<= nm.msortf(f=para,o="%s/pair_%s.csv"%(self.outDir,iter))
				f.run()
				
			# 終了判定
			if(iter>=self.iterMax):
				break

			if iter!=0 and self.same(xxtra,xxprev):
				break

			shutil.copyfile(xxtra,xxprev)

			tpstr = self.measure+"_"		if self.msgoff else self.measure

			extTake.sspc(type=tpstr,l=self.minSupp,i=xxtra,th=self.th,o=xxpair)


			#gtpstr = "ue0_" if self.msgoff else "ue0"
			#extTake.grhfil(type=gtpstr,i=xxpair,o=xxtra)

			iter+=1


		# 上記iterationで収束したマイクロクラスタグラフを元の節点文字列に直して出力する
		os.system("tr ' ' ',' < %s > %s"%(xxpair,xxtmmp))
		f = nm.mcut(f="0:num1,1:num2",nfni=True,i=xxtmmp)
		f <<= nm.mjoin(k="num1",K="id",m=xxmaprev,f="node:%s"%(self.gi.edgeFN1)).mjoin(k="num2",K="id",m=xxmaprev,f="node:%s"%(self.gi.edgeFN2))
		f <<= nm.mcut(f=para).mfsort(f=para).msortf(f=para,o=self.__eo)
		f.run()

		if(self.__no):
			if self.__nf:
				nm.mcut(f="node:%s"%(self.__nf),i=xxmap,o=self.__no).run()
			else:
				nm.mcut(f="node",i=xxmap,o=self.__no).run()

		if(self.go):
			self.go.readCSV(edgeFile=self.__eo,title1=self.gi.edgeFN1,title2=self.gi.edgeFN2,nodeFile=self.__no,title=self.__nf)
		else:
			self.go = ntg.graph(edgeFile=self.__eo,title1=self.gi.edgeFN1,title2=self.gi.edgeFN2,nodeFile=self.__no,title=self.__nf)

		procTime = datetime.now()-t
		# ログファイル出力
		if(self.logFile):
			kv=[["key","value"]]
			
			kv.append(["iter",str(self.iterMax)])
			kv.append(["outDir",str(self.outDir)])
			kv.append(["th",str(self.th)])
			kv.append(["indirect",str(self.indirect)])
			kv.append(["measure",str(self.measure)])

			kv.append(["minSupp",str(self.minSupp)])
			kv.append(["logFile",str(self.logFile)])
			kv.append(["outDir",str(self.outDir)])


			kv.append(["time",str(procTime)])
			for i in range(len(nSizes)):
				kv.append(["nSize"+str(i),str(nSizes[i])]);
				kv.append(["eSize"+str(i),str(eSizes[i])]);
				kv.append(["dens"+str(i) ,str(denses[i])]);

			nm.writecsv(i=kv,o=self.logFile).run()

		nu.mmsg.endLog(self.__cmdline())

		return self.go
	 



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs
import nysol.take as nt


args=margs.Margs(sys.argv,"i=,tid=,item=,ro=,eo=,no=,s=,S=,filter=,lb=,ub=,sim=,dir=,rank=,-prune,-num,-verbose,-rp","i=,tid=,item=,eo=,no=")
nt.mpal(**args.kvmap()).run(msg="on")




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Mnewrand(NysolMOD_CORE):

	_kwd ,_inkwd,_outkwd = n_core.getparalist("mnewrand",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_Mnewrand,self).__init__("mnewrand",nutil.args2dict(args,kw_args,Nysol_Mnewrand._kwd))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs 
import nysol.take as nt

args=margs.Margs(sys.argv,"ni=,nf=,ei=,ef=,eo=,no=,sim=,rank=,dir=,-directed,-udout,-rp","ei=,ef=,sim=")
nt.mfriends(**(args.kvmap())).run(msg="on")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/
import os
import shutil
import nysol.util.mtemp as mtemp
import nysol.util.mrecount as mrecount
import nysol.mcmd as nm
from nysol.take import extcore as extTake
import nysol.vsop._vsoplib as VSOP

#========================================================================
# 列挙関数:lcm 利用DB:TraDB
#========================================================================
class LcmIs(object):
	"""
	:type db: nysol.take.traDB.TraDBオブジェクト
	:param db: transactionデータベース名

	トランザクションデータベースから頻出アイテム集合を列挙するクラス。

	例1) 最小サポート件数が3件、最小アイテムサイズが2件の頻出アイテム集合を列挙し、その後に同条件の頻出飽和集合を求める。

	.. code-block:: python
		:linenos:

		import os
		from nysol.take.traDB import TraDB
		from nysol.take.enumLcmIs import LcmIs

		d='''id,item
		"T1","C"
		"T1","E"
		"T2","D"
		"T2","E"
		"T2","F"
		"T3","A"
		"T3","B"
		"T3","D"
		"T3","A"
		"T4","B"
		"T4","D"
		"T4","F"
		"T5","A"
		"T5","B"
		"T5","D"
		"T5","E"
		"T6","A"
		"T6","B"
		"T6","D"
		"T6","E"
		"T6","F"
		'''

		with open("dat1.csv","w") as fpw:
	  		fpw.write(d)

				db=TraDB(iFile="dat1.csv",idFN="id",itemFN="item")

				lcm=LcmIs(db)

		eArgs={}
		eArgs["type"]="F"
		eArgs["minCnt"]=3
		eArgs["minLen"]=2
		lcm.enumerate(eArgs)
		oPath="./xxrsl1"
		os.system("mkdir %s"%oPath)
		lcm.output(oPath)
		print(lcm)

	.. code-block:: python
		:linenos:

		## class name: nysol.take.enumLcmIs.LcmIs
		id: 10805b4a8
		eArgs: {'type': 'F', 'minCnt': 3, 'minLen': 2}
		minCnt: 3
		minSup: 0.5
		maxCnt: None
		maxSup: None
		minLen: None
		maxLen: None
		top: None
		skipTP: False

		## class name: nysol.take.enumLcmIs.LcmIs
		id: 10805b4a8
		eArgs: {'type': 'C', 'minCnt': 3, 'minLen': 2}
		minCnt: 3
		minSup: 0.5
		maxCnt: None
		maxSup: None
		minLen: None
		maxLen: None
		top: None
		skipTP: False
	"""
	def __init__(self,db):
		self.db=db
		self.eArgs=None
		self.type =None
		self.minCnt=None
		self.minSup=None
		self.maxCnt=None
		self.maxSup=None
		self.minLen=None
		self.maxLen=None
		self.top =None
		self.skipTP=False

		#self.size =None
		self.pFile =None
		self.tFile =None
		self.msgoff = True

		self.temp=mtemp.Mtemp()
		self.db = db # 入力データベース
		self.file=self.temp.file()
		items=self.db.items

		# アイテムをシンボルから番号に変換する。
		f =   nm.mjoin(k=self.db.itemFN,K=items.itemFN,m=items.file,f=items.idFN,i=self.db.file)
		f <<= nm.mcut(f=self.db.idFN+","+items.idFN)
		f <<= nm.mtra(k=self.db.idFN,f=items.idFN)
		f <<= nm.mcut(f=items.idFN,nfno=True,o=self.file)
		f.run()

	def reduceTaxo(self,pat,items):
		#tf=MCMD::Mtemp.new

		if items.taxonomy==None:
			return pat

		taxo=items.taxonomy
		xxrt = nm.mtrafld(f=taxo.itemFN+","+taxo.taxoFN,valOnly=True,a="__fld",i=taxo.file).mcut(f="__fld")
		# xxrtの内容：oyakoに親子関係にあるアイテム集合のリストが格納される
		# __fld
		# A X
		# B X
		# C Y
		# D Z
		# E Z
		# F Z

		oyako=VSOP.constant(0)
		for fldVal in xxrt: 
			oyako=oyako+VSOP.itemset(fldVal[0])

		#print("-------")
		#pat.Print()
		pat=pat.restrict(oyako).iif(0,pat)
		#print("-------")
		#pat.Print()

		return pat

	def __str__(self):
		ret ="## class name: "+self.__module__+"."+self.__class__.__name__+"\n"
		ret+="id: "+"%x"%id(self)+"\n"
		ret+="eArgs: "+self.eArgs.__str__()+"\n"
		ret+="minCnt: "+str(self.minCnt)+"\n"
		ret+="minSup: "+str(self.minSup)+"\n"
		ret+="maxCnt: "+str(self.maxCnt)+"\n"
		ret+="maxSup: "+str(self.maxSup)+"\n"
		ret+="minLen: "+str(self.minLen)+"\n"
		ret+="maxLen: "+str(self.maxLen)+"\n"
		ret+="top: "   +str(self.top)+"\n"
		ret+="skipTP: "+str(self.skipTP)+"\n"
		#ret+="db: "+self.db.__str__()+"\n"
		#ret+="size: "  +str(self.size)+"\n"
		return ret

	def enumerate(self,eArgs):
		"""
		eArgsで与えられた条件で、頻出アイテム集合の列挙を実行する。

		:type eArgs: dict
		:type eArgs['type']: str
		:type eArgs['minCnt']: int
		:type eArgs['minSup']: float
		:type eArgs['maxCnt']: int
		:type eArgs['maxSup']: float
		:type eArgs['minLen']: int
		:type eArgs['maxLen']: int
		:type eArgs['top']: int
		:type eArgs['skipTP']: bool【default:False】
		:param eArgs: 各種列挙パラメータ
		:param eArgs['type']: 抽出するアイテム集合の型【'F':頻出集合, 'C':飽和集合, 'M':極大集合】
		:param eArgs['minCnt']: 最小サポート(件数)
		:param eArgs['minSup']: 最小サポート(確率)
		:param eArgs['maxCnt']: 最大サポート(件数)
		:param eArgs['maxSup']: 最大サポート(確率)
		:param eArgs['minLen']: アイテム集合の最小アイテム数(件数)
		:param eArgs['maxLen']: アイテム集合の最大アイテム数(件数)
		:param eArgs['top']: 列挙するサポート上位件数(件数)
		:param eArgs['skipTP']: トランザクションにマッチするパターン(アイテム集合)の出力を行わない。
		"""

		tf=mtemp.Mtemp()
		self.eArgs=eArgs
		self.type = eArgs["type"]

		if "minCnt" in eArgs and eArgs["minCnt"] != None:
			self.minCnt = int(eArgs["minCnt"])
			self.minSup = float(self.minCnt) / float(self.db.traSize)
		else:
			self.minSup = float(eArgs["minSup"])
			self.minCnt = int(self.minSup * float(self.db.traSize) + 0.99)

		# 最大サポートと最大サポート件数
		self.maxCnt=None
		if ("maxCnt" in eArgs and  eArgs["maxCnt"]!= None) or ( "maxSup" in eArgs and eArgs["maxSup"]!= None):
			if "maxCnt" in eArgs and eArgs["maxCnt"]!= None:
				self.maxCnt = int(eArgs["maxCnt"])
				self.maxSup = float(self.maxCnt) / float(self.db.traSize)
			else:
				self.maxSup    = float(eArgs["maxSup"])
				self.maxCnt = int(self.maxSup * float(self.db.traSize) + 0.99)


		params = {}
		if self.msgoff:
			params["type"] ="%sIf_"%(self.type)
		else:
			params["type"] ="%sIf"%(self.type)


		if self.maxCnt :
			params["U"] = str(self.maxCnt)

		if "minLen" in eArgs and eArgs["minLen"] != None :
			params["l"] = str(eArgs['minLen'])
		
		if "maxLen" in eArgs and eArgs["maxLen"] != None :
			params["u"] = str(eArgs['maxLen'])

		# 列挙パターン数上限が指定されれば、一度lcmを実行して最小サポートを得る
		if "top" in eArgs and eArgs["top"] != None :
			self.top = eArgs["top"]

		if self.top and self.top>0 :

			xxtop = tf.file()
			import copy
			top_params = copy.deepcopy(params)
			top_params["i"] = self.file
			top_params["sup"] = "1"
			top_params["K"] = str(self.top)
			top_params["so"] = xxtop
			import re
			top_params["type"] = re.sub('_$', '', top_params["type"] )

			extTake.lcm(top_params)

			with open(xxtop, "r") as rfile:
				self.minCnt = int(rfile.read().strip())

			if self.minCnt<0 :
				self.minCnt=1 


		self.skipTP=False
		if "skipTP" in eArgs:
			self.skipTP=eArgs["skipTP"]

		# lcm_seq出力ファイル
		lcmout = tf.file()

		# 頻出パターンがなかった場合、lcm出力ファイルが生成されないので
		# そのときのために空ファイルを生成しておいく。
		with open(lcmout, "w") as efile:
			pass

		# lcm実行
		params["i"] = self.file
		params["sup"] = str(self.minCnt)
		params["o"] = lcmout
		extTake.lcm(params)

		# caliculate one itemset for lift value
		xxone= tf.file()
		tpstr = "FIf_" if self.msgoff else "FIf"

		extTake.lcm(type=tpstr,i=self.file,sup=1,o=xxone,l=1,u=1)


		# パターンのサポートを計算しCSV出力する
		#MCMD::msgLog("output patterns to CSV file ...")

		xxp0 = tf.file()
		self.pFile = self.temp.file()
		items=self.db.items
		trans0 = self.temp.file()

		extTake.lcmtrans(lcmout,"p",trans0)

		f =   nm.mdelnull(i=trans0,f="pattern")
		f <<= nm.mvreplace(vf="pattern",m=items.file,K=items.idFN,f=items.itemFN)
		f <<= nm.msetstr(v=self.db.traSize,a="total")
		f <<= nm.mcal(c='${count}/${total}',a="support")
		f <<= nm.mcut(f="pid,pattern,size,count,total,support")
		f <<= nm.mvsort(vf="pattern")
		f <<= nm.msortf(f="pid",o=xxp0)
		f.run()


		# xxp0
		# pid,count,total,support,pattern
		# 0,13,13,1,A
		# 4,6,13,0.4615384615,A B
		xxp1=tf.file()

		# taxonomy指定がない場合(2010/11/20追加)
		if items.taxonomy==None:
			shutil.move(xxp0,xxp1)
		# taxonomy指定がある場合
		else:
			#MCMD::msgLog("reducing redundant rules in terms of taxonomy ...")

			zdd=VSOP.constant(0)
			fobj = nm.mcut(i=xxp0,f='pattern')
			for fldVal in fobj:
				zdd=zdd+VSOP.itemset(fldVal[0])

			
			zdd=self.reduceTaxo(zdd,self.db.items)
			xxz1=tf.file()
			xxz2=tf.file()
			zdd.csvout(xxz1)

			f0=None
			f0 <<= nm.mcut(nfni=True,f="1:pattern",i=xxz1)
			f0 <<= nm.mvsort(vf="pattern")
			f0 <<= nm.msortf(f="pattern")

			f=None
			f <<= nm.msortf(f="pattern",i=xxp0)
			f <<= nm.mcommon(k="pattern",m=f0)
			f <<= nm.msortf(f="pid",o=xxp1)
			f.run()


		# lift値の計算		
		transl = tf.file()
		extTake.lcmtrans(xxone,"p",transl)

		xxp2 =   nm.mdelnull(i=transl,f="pattern")
		xxp2 <<= nm.mvreplace(vf="pattern",m=items.file,K=items.idFN,f=items.itemFN)
		xxp2 <<= nm.msortf(f="pattern")

		xxp3 =   nm.mcut(f="pid,pattern",i=xxp1)
		xxp3 <<= nm.mtra(f="pattern",r=True)
		xxp3 <<= nm.mjoin(k="pattern",m=xxp2,f="count:c1")
		xxp3 <<= nm.mcal(c='ln(${c1})',a="c1ln")
		xxp3 <<= nm.msum(k="pid",f="c1ln")

		# p3
		# pid,pattern,c1,c1ln
		# 0,A,13,2.564949357
		# 1,E,7,1.945910149
		
		#おかしくなる?=>OK
		f3 =   nm.mjoin(k="pid",f="c1ln",i=xxp1,m=xxp3)
		f3 <<= nm.mcal(c='round(exp(ln(${count})-${c1ln}+(${size}-1)*ln(${total})),0.0001)',a="lift")
		f3 <<= nm.mcut(f="pid,size,count,total,support,lift,pattern")
		f3 <<= nm.msortf(f="support%nr",o=self.pFile)
		f3.run()

		#self.size = mrecount.mrecount(i=self.file)

		#MCMD::msgLog("the number of patterns enumerated is #{@size}")

		if not self.skipTP:
			# トランザクション毎に出現するシーケンスを書き出す
			#MCMD::msgLog("output tid-patterns ...")

			self.tFile = self.temp.file()
			xxw3i = tf.file()
			extTake.lcmtrans(lcmout,"t",xxw3i)

			xxw1 = nm.mcut(f=self.db.idFN,i=self.db.file).muniq(k=self.db.idFN).mnumber(S=0,a="__tid",q=True).msortf(f="__tid")
			xxw2 = nm.mcut(f="pid",i=self.pFile)
			xxw3 = nm.mcommon(k="pid",i=xxw3i,m=xxw2).mjoin(k="__tid",m=xxw1,f=self.db.idFN).mcut(f=self.db.idFN+",pid",o=self.tFile)
			xxw3.run()

	def output(self,outpath):
		"""
		列挙された結果を指定のパス(outpath)に保存する。

		:type outpath: str
		:param outpath: 結果を出力するパス名。存在しなければエラーとなるので、事前にディレクトリを作成しておくこと。
		"""
		nm.mfldname(q=True,i=self.pFile,o=outpath+"/patterns.csv").run()
		#shutil.move(self.pFile,outpath+"/patterns.csv")
		if not self.skipTP:
			nm.mfldname(q=True,i=self.tFile,o=outpath+"/tid_pats.csv").run()
			#shutil.move(self.tFile,outpath+"/tid_pats.csv")
  		



#!/usr/bin/env ruby
# encoding: utf-8
import os
import nysol.mcmd as nm
import nysol.util.mtemp as mtemp

#=分類階層(taxonomy)クラス
# 一階層の分類階層(アイテム-分類)を扱うクラス。
#
#===利用例
# 以下taxo.csvの内容
# ---
# item,taxo
# a,X1
# b,X1
# c,X2
# d,X2
#
# 以下rubyスクリプト
# ---
# require 'rubygems'
# require 'mining'
# taxo=Taxonomy("taxo.csv","item","taxo")
# puts taxo.itemFN   # => "item"
# puts taxo.taxoFN   # => "taxo"
# puts taxo.itemSize # => 4 (アイテムは"a,b,c,d"の4種類)
# puts taxo.taxoSize # => 2 (分類は"X1,X2"の2種類)
# puts taxo.file     # => ./1252379737_1756/dat/1
#
#=====./1252379737_1756/dat/1 の内容
# item,taxo
# a,X1
# b,X1
# c,X2
# d,X2
class Taxonomy(object):


	#=== taxonomyクラスの初期化
	# 一階層の分類階層を扱う。
	# アイテム項目とその分類名項目を持つファイルから分類階層オブジェクトを生成する。
	#====引数
	# iFile: taxonomyファイル名
	# itemFN: アイテム項目名
	# taxoFN: 分類項目名
	#====機能
	#* アイテム(itemFN)と分類(taxoFN)の2項目からなるファイルが規定のパス(Taxonomy.file)に書き出される。
	#* 同じアイテムが重複して登録されていれば単一化して書き出される。
	#* アイテム順にソートされる。
	#* アイテム数と分類数を計算する。
	def __init__(self,iFile,itemFN,taxoFN):
		# アイテムの項目名(=>String)
		self.itemFN = None

		# 分類の項目名(=>String)
		self.taxoFN = None

		# アイテムの種類数(=>Fixnum)
		self.itemSize = None

		# 分類の種類数(=>Fixnum)
		self.taxoSize = None

		# taxonomyデータファイル名(=>String)
		self.file = None

		self.temp = mtemp.Mtemp()

		self.iFile  = iFile

		self.iPath  = os.path.abspath(self.iFile)
		self.itemFN  = itemFN
		self.taxoFN  = taxoFN

		# item順に並べ替えてpathに書き出す
		self.file = self.temp.file()
		para_it = self.itemFN +"," + self.taxoFN
		nm.mcut(f=para_it,i=self.iFile).muniq(k=para_it,o=self.file).run(msg="on")
	

		f = nm.mcut(f=self.itemFN,i=self.iFile)
		f <<= nm.mtrafld(f=self.itemFN,a="__fld",valOnly=True)
		f <<= nm.mtra(f="__fld",r=True)
		f <<= nm.muniq(k="__fld")
		f <<= nm.mcount(a="size")
		f <<= nm.mcut(f="size")
		xx1 = f.run()

		self.itemSize = int(xx1[0][0])

		xx2 = nm.mcut(f=self.taxoFN+":item",i=self.file).muniq(k="item").mcount(a="size").mcut(f="size").run()
		self.taxoSize = int(xx2[0][0])





# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

def _para2dict(func,args,kw_args):
	params = {}
	params["runfunc_func"] = func 
	params["runfunc_args"] = args
	params["runfunc_kwd"]  = kw_args
	return params


class Nysol_Runfunc(NysolMOD_CORE):

	_kwd,_inkwd,_outkwd  = n_core.getparalist("runfunc",3)
	_disabled_ouputlist = True

	def __init__(self,func,*args, **kw_args) :
		super(Nysol_Runfunc,self).__init__("runfunc",_para2dict(func,args,kw_args))


def runfunc(self,func,*args, **kw_args):
	return Nysol_Runfunc(_para2dict(func,args,kw_args)).addPre(self)

setattr(NysolMOD_CORE, "runfunc", runfunc)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import nysol.mcmd as nm
import nysol.util as nu

from nysol.take import extcore as extTake

class mtra2gc(object):

	helpMSG="""
# ver="1.0" # 初期リリース 2016/11/20
# ver="1.1" # resemblanceをjaccardに変更 2016/12/13
# ver="1.2" # sim=の値を変更 2016/12/13
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) トランザクションデータからアイテム類似グラフを構築する。
内容) 2アイテムの共起情報によって類似度を定義し、ある閾値より高い類似度を持つアイテム間に枝を張る。
      mtra2g.rbで可能なclassやtaxonomyの指定は出来ないが、より高速に動作する。
      また類似度の定義にconfidenceを指定可能。
書式) #{$cmd} i= tid= item= [class=] [no=] eo= [s=|S=] [sim=] [th=] [-node_support] [-num] [log=] [T=] [--help] 

  ファイル名指定
  i=     : トランザクションデータファイル【必須】
  tid=   : トランザクションID項目名【必須】
  item=  : アイテム項目名【必須】
  no=    : 出力ファイル(節点)
  eo=    : 出力ファイル(辺:節点ペア)
  log=   : パラメータの設定値をkey-value形式のCSVで保存するファイル名

  【枝を張る条件1】
  s=     : 最小支持度(全トランザクション数に対する割合による指定): 0以上1以下の実数
  S=     : 最小支持度(トランザクション数による指定): 1以上の整数
           S=,s=両方共省略時はs=0.01をデフォルトとする

  【枝を張る条件2:省略可】
  sim=   : アイテムa,bに枝を張る条件として用いる類似度を指定する。
           省略した場合は、最小支持度の条件でのみ枝を張ることになる。
           指定できる類似度は以下の3つのいずれか一つ。
           省略した場合はs=もしくはS=の条件のみで実行される。
             J (jaccard)              : |A ∩ B|/|A ∪ B|
             P (normalized PMI)       : log(|A ∩ B|*|T| / (|A|*|B|)) / log(|A ∩ B|/|T|)
                                        liftを-1〜+1に基準化したもの。
                                        -1:a(b)出現時b(a)出現なし、0:a,b独立、+1:a(b)出現時必ずb(a)出現
             C (Confidence(A=>B))     : |A ∩ B|/|B|
                      A,B: アイテムa(b)を含むトランザクション集合
                      T: 全トランザクション集合
  th=    : sim=で指定された類似度について、ここで指定された値以上のアイテム間に枝を張る。

  【節点条件】
  -node_support : 節点にもS=の条件を適用する。指定しなければ全てのitemを節点として出力する。

  その他
  -num : アイテム項目が正の整数値である場合に指定可能で、処理が高速化される。
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

入力ファイル形式)
トランザクションIDとアイテムの２項目によるトランザクションデータ。
class=を指定する場合は、さらにクラス項目が必要となる。
使用例を参照のこと。

出力形式)
a) 節点ファイル(no=)
例:
  node%0,support,frequency,total
  a,0.6,3,5
  b,0.8,4,5
  c,0.2,1,5
  d,0.8,4,5
  e,0.4,2,5
  f,0.8,4,5
項目の説明:
  node:アイテム
  support:frequency/total
  frequency:アイテムの出現頻度
  total:全トランザクション数

b) 枝ファイル(eo=)
例:
  node1%0,node2%1,frequency,frequency1,frequency2,total,support,confidence,lift,jaccard,PMI
  a,b,3,3,4,5,0.6,1,1.25,0.75,0.4368292054
  a,c,1,1,3,5,0.2,1,1.666666667,0.3333333333,0.3173938055
項目の説明:
  node1,node2:アイテム
  support:frequency/total
  frequency:2つのアイテム(node1,node2)の共起頻度
  frequency1:node1の出現頻度
  frequency2:node2の出現頻度
  total:全トランザクション数
  confidence: frequency/frequency1
  lift: (total*frequency)/(frequency1*frequency2)
  jaccard,PMI:上述の「枝を張る条件2」を参照

基本的な使用例)
$ cat tra1.csv 
id,item
1,a
1,b
1,c
1,f
2,d
2,e
2,f
3,a
3,b
3,d
3,f
4,b
4,d
4,f
5,a
5,b
5,d
5,e
$ #{$cmd} i=tra.csv tid=id item=item S=1 sim=C th=0.7 no=node.csv eo=edge.csv
$ cat node.csv 
node%0,support,frequency,total
a,0.6,3,5
b,0.8,4,5
c,0.2,1,5
d,0.8,4,5
e,0.4,2,5
f,0.8,4,5
$ cat edge.csv
node1%0,node2%1,frequency,frequency1,frequency2,total,support,confidence,lift,jaccard,PMI
a,b,3,3,4,5,0.6,1,1.25,0.75,0.4368292054
b,a,3,4,3,5,0.6,0.75,1.25,0.75,0.4368292054
b,d,3,4,4,5,0.6,0.75,0.9375,0.6,-0.1263415893
b,f,3,4,4,5,0.6,0.75,0.9375,0.6,-0.1263415893
c,a,1,1,3,5,0.2,1,1.666666667,0.3333333333,0.3173938055
c,b,1,1,4,5,0.2,1,1.25,0.25,0.1386468839
c,f,1,1,4,5,0.2,1,1.25,0.25,0.1386468839
d,b,3,4,4,5,0.6,0.75,0.9375,0.6,-0.1263415893
d,f,3,4,4,5,0.6,0.75,0.9375,0.6,-0.1263415893
e,d,2,2,4,5,0.4,1,1.25,0.5,0.2435292026
f,b,3,4,4,5,0.6,0.75,0.9375,0.6,-0.1263415893
f,d,3,4,4,5,0.6,0.75,0.9375,0.6,-0.1263415893

# Copyright(c) NYSOL 2012- All Rights Reserved.
		"""

	verInfo="version=1.2"


	paramter = {	
		"i":"filename",
		"no":"filename",
		"eo":"filename",
		"log":"filename",
		"tid":"fld",
		"item":"fld",
		"s":"int",
		"S":"int",
		"sim":"str",
		"th":"float",
		"node_support":"bool",
		"T=":"str",
		"rp":"bool",
		"num":"bool"
	}

	paramcond = {	
		"hissu": ["i","tid","item","eo"]
	}

	def help():
		print(mtra2gc.helpMSG) 

	def ver():
		print(mtra2gc.versionInfo)

	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in mtra2gc.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )
			#型チェック入れる

		self.msgoff = True

		self.iFile   = kwd["i"]
		self.oeFile  = kwd["eo"]
		self.idFN    = kwd["tid"]  
		self.itemFN  = kwd["item"] 

		self.onFile  = kwd["no"]        if "no"  in kwd else None
		self.logFile = kwd["log"]       if "log" in kwd else None
		self.sim     = kwd["sim"]       if "sim" in kwd else None
		self.th      = float(kwd["th"]) if "th"  in kwd else None # 類似度measure
		self.rpf  = kwd["rp"] if "rp" in kwd else False 

		self.node_support = kwd["node_support"] if "node_support" in kwd else False
		self.num = kwd["num"] if "num" in kwd else False

		# 最小サポート件数
		self.minSupPrb = float(kwd["s"])  if "s" in kwd else None
		self.minSupCnt = float(kwd["S"])  if "S" in kwd else None

		if self.minSupPrb==None and self.minSupCnt==None :
			self.minSupPrb=0.01 

		if self.sim and "JPC".find(self.sim)==-1:
			raise Exception("sim= takes one of 'J','P','C'")



	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 


	def __init__(self,**kwd):

		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)


	def convN(self,iFile,idFN,itemFN,oFile,mapFile):


		f0 = nm.mcut(f=itemFN+":##item",i=iFile).mcount(k="##item",a="##freq",o=mapFile)
		f0.run()

		extTake.mmaketra(i=iFile,o=oFile,f=itemFN,k=idFN)
		#f1 = nm.mtra(k=idFN,f=itemFN+":##num",i=iFile).mcut(f="##num",nfno=True,o=oFile)


		#nm.runs([f0,f1])
		# #{mapFile}"
		# ##item,##freq%0nr,##num
		# b,4,0
		# d,4,1
		return nu.mrecount(i=oFile,nfn=True)

	def conv(self,iFile,idFN,itemFN,oFile,mapFile):
		temp=nu.Mtemp()
		xxtra=temp.file()

		# 入力ファイルのidがnilの場合は連番を生成して新たなid項目を作成する。
		f0   = nm.mcut(f=itemFN+":##item",i=iFile)
		f0 <<= nm.mcount(k="##item",a="##freq")
		f0 <<= nm.mnumber(s="##freq%nr",a="##num",o=mapFile)
		f0.run()

		f1   = nm.mjoin(k=itemFN,K="##item",f="##num",m=mapFile,i=iFile,o=xxtra)
		f1.run()
		
		#f1 <<= nm.mtra(k=idFN,f="##num")
		#f1 <<= nm.mnumber(q=True,a="##traID")
		#f1 <<= nm.mcut(f="##num",nfno=True,o=oFile)

		extTake.mmaketra(i=xxtra,o=oFile,f="##num",k=idFN)

		#f1.run()

		# ##item,##freq%0nr,##num
		# b,4,0
		# d,4,1
		size=nu.mrecount(i=oFile,nfn=True)

		return size


	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"
				
		 

		from datetime import datetime	
		t = datetime.now()

		temp=nu.Mtemp()
		xxsspcin=temp.file()
		xxmap=temp.file()

		# traファイルの変換とマップファイルの作成
		if self.num :
			total = self.convN(self.iFile,self.idFN,self.itemFN,xxsspcin,xxmap)
		else:
			total = self.conv(self.iFile,self.idFN,self.itemFN,xxsspcin,xxmap)

		# system "head xxsspcin"
		# 3 5 0 2
		# 4 1 2
		# 0 2 3 1
		# 1 0 2
		# 3 4 0 1
		# system "head xxmap"
		# ##item,##freq%0nr,##num
		# b,4,0
		# d,4,1
		# f,4,2
		minSupp = int(total*self.minSupPrb)	if self.minSupPrb else self.minSupCnt
			

		# sspc用simの文字列
		if self.sim :
			if self.sim=="J":
				sspcSim="R"
			elif self.sim=="P":
				sspcSim="P"
			elif self.sim=="C":
				sspcSim="i"
		# sim=省略時はRでth=0とする(sim制約なし)
		else:
			sspcSim="R"
			self.th=0

		############ 列挙本体 ############
		xxsspcout=temp.file()
		tpstr =  sspcSim+"ft_" if self.msgoff else sspcSim+"ft"
		extTake.sspc(type=tpstr,TT=minSupp,i=xxsspcin,th=self.th,o=xxsspcout)

		##################################

		xxtmmp=temp.file()
		
		f =   nm.mread(i=xxsspcout) 
		f <<= nm.cmd("tr ' ()' ','") 
		f <<= nm.mcut(f="1:i1,2:i2,0:frequency,4:sim",nfni=True)

		if self.num :

			f <<= nm.mfldname(f="i1:node1,i2:node2")
			if self.sim!="C":
				f <<= nm.mfsort(f="node1,node2")
			
			f <<= nm.mjoin(k="node1",K="##item",m=xxmap,f="##freq:frequency1")
			f <<= nm.mjoin(k="node2",K="##item",m=xxmap,f="##freq:frequency2") 
			
		else:

			f <<= nm.mjoin(k="i1",K="##num",m=xxmap,f="##item:node1,##freq:frequency1")
			f <<= nm.mjoin(k="i2",K="##num",m=xxmap,f="##item:node2,##freq:frequency2") 

			if self.sim!="C":

				f <<= nm.mcut(f="i1,i2,frequency,sim,node1,node2,frequency1,frequency2,node1:node1x,node2:node2x")
				f <<= nm.mfsort(f="node1x,node2x")
				f <<= nm.mcal(c='if($s{node1}==$s{node1x},$s{frequency1},$s{frequency2})',a="freq1")
				f <<= nm.mcal(c='if($s{node2}==$s{node2x},$s{frequency2},$s{frequency1})',a="freq2")
				f <<= nm.mcut(f="i1,i2,frequency,sim,node1x:node1,node2x:node2,freq1:frequency1,freq2:frequency2")

		f <<= nm.msetstr(v=total,a="total")
		f <<= nm.mcal(c='${frequency}/${frequency1}',a="confidence")
		f <<= nm.mcal(c='${frequency}/${total}',a="support")
		f <<= nm.mcal(c='${frequency}/(${frequency1}+${frequency2}-${frequency})',a="jaccard")
		f <<= nm.mcal(c='(${frequency}*${total})/((${frequency1}*${frequency2}))',a="lift")
		f <<= nm.mcal(c='(ln(${frequency})+ln(${total})-ln(${frequency1})-ln(${frequency2}))/(ln(${total})-ln(${frequency}))',a="PMI")
		f <<= nm.mcut(f="node1,node2,frequency,frequency1,frequency2,total,support,confidence,lift,jaccard,PMI")
		f <<= nm.msortf(f="node1,node2",rp=self.rpf ,o=self.oeFile)
		
		
		f.run()

		if self.onFile:
			f4 =   nm.mcut(f=self.itemFN+":node",i=self.iFile)
			f4 <<= nm.mcount(k="node",a="frequency")
			if self.node_support :
				minstr = "[%s,]"%(minSupp)
				f4 <<= nm.mselnum(f="frequency",c=minstr)

			f4 <<= nm.msetstr(v=total,a="total")
			f4 <<= nm.mcal(c='${frequency}/${total}',a="support")
			f4 <<= nm.mcut(f="node,support,frequency,total",rp=self.rpf ,o=self.onFile)
			f4.run()

		procTime=datetime.now()-t

		# ログファイル出力
		if self.logFile :
			kv=[["key","value"]]
			for k,v in self.args.items():
				kv.append([k,str(v)])
			kv.append(["time",str(procTime)])
			nm.writecsv(i=kv,o=self.logFile).run()

		nu.mmsg.endLog(self.__cmdline())





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
	==================================
	Takeライブラリのコアアルゴリズム
	==================================
	C言語で開発されたTakeライブラリのコアアルゴリズムをpythonから利用できるようにラッピングしたものである。
	オリジナルのコードは http://research.nii.ac.jp/~uno/codes-j.htm より入手できる。

	:py:func:`lcm` は・・・

	:ref:`lcmへ<lcm-label>`

	:py:func:`take.extcore.lcm` は・・・


	* lcm
	* lcmseq


	xxx title
	-------------------

	extcoreパッケージが使うデータ形式を書く。
	トランザクション形式
	グラフエッジ形式
	グラフ隣接行列形式

	最終更新日: |today|

"""
from nysol.take import _grhfillib as lib_grhfil
from nysol.take import _lcmlib as lib_lcm
from nysol.take import _lcmseq_zerolib as lib_lcmseq0
from nysol.take import _lcmseqlib  as lib_lcmseq
from nysol.take import _lcmtranslib  as lib_lcmtarns
from nysol.take import _mmaketralib  as lib_mmaketra

from nysol.take import _macelib as lib_mace
from nysol.take import _medsetlib as lib_medset
from nysol.take import _simsetlib as lib_simset
from nysol.take import _sspclib as lib_sspc

def _kwd2list(args, kw_args,paraConf,extpara=None):
	def tostr(val):
		if isinstance(val,list):
			newlist=[]
			for cval in val:
				newlist.append(str(cval))
			if len(newlist) != 0 :
				return ",".join(newlist)
			else:
				return None
		else:
			return str(val)

	if len(args)>1:
		raise Exception("args zero or one ")

	if len(args)==1 and not isinstance(args[0],dict) :
		raise Exception("args is not dict")

	# dict Merge		
	if len(args)==1:
		kw_args.update(args[0])

	# 可能パラメータ設定
	enablePara = set([])
	for v in paraConf.values():
		for vv in v:
			enablePara.add(vv[0])

	# 可能パラメータCHECK
	for k in kw_args.keys():
		if not k in enablePara:
			raise Exception("Unknown parameter")

	newargs=[]
	if "saki" in paraConf:
		for paraInfo in paraConf["saki"]:
			if paraInfo[0] in kw_args:
				newargs.append(tostr(kw_args[paraInfo[0]]))
			elif paraInfo[1] == 1:
				raise Exception("not find necceary parameter")
				
	if "opt" in paraConf:
		for paraInfo in paraConf["opt"]:
			if paraInfo[0] in kw_args:
				newargs.append(tostr(paraInfo[1]))
				newargs.append(tostr(kw_args[paraInfo[0]]))

	if "opts" in paraConf:
		for paraInfo in paraConf["opts"]:

			if paraInfo[0] in kw_args:

				newargs.append(tostr(paraInfo[1]))

				if isinstance(kw_args[paraInfo[0]],list):
					if len(kw_args[paraInfo[0]])!=paraInfo[2] :
						raise Exception("not valid parameter")
					for val in kw_args[paraInfo[0]]:
						newargs.append(tostr(val))

				elif isinstance(kw_args[paraInfo[0]],str):
					optsub = kw_args[paraInfo[0]].split(",") 
					if len(optsub)!=paraInfo[2] :
						raise Exception("not valid parameter")
					for val in optsub:
						newargs.append(tostr(val))
				
				else:
					raise Exception("not valid parameter")


	if "addkw" in paraConf:
		for paraInfo in paraConf["addkw"]:

			if paraInfo[0] in kw_args:

				if isinstance(kw_args[paraInfo[0]],list):

					if len(kw_args[paraInfo[0]])==1:
						newargs.append( tostr(paraInfo[1]) )
					elif len(kw_args[paraInfo[0]])==2:
						newargs.append( tostr(paraInfo[1])+tostr(kw_args[paraInfo[0]][1]))
					else:
						raise Exception("not valid parameter")

					newargs.append(tostr(kw_args[paraInfo[0]][0]))


				elif isinstance(kw_args[paraInfo[0]],str):
					optsub = kw_args[paraInfo[0]].split(",") 
					if len(optsub)==1:
						newargs.append(tostr(paraInfo[1]) )
					elif len(optsub)==2:
						newargs.append(tostr(paraInfo[1])+tostr(optsub[1]) )
					else:
						raise Exception("not valid parameter")

					newargs.append(tostr(optsub[0]))
				
				else:
					raise Exception("not valid parameter")

	if "optB" in paraConf:
		for paraInfo in paraConf["optB"]:
			if paraInfo[0] in kw_args:
				if kw_args[paraInfo[0]]:
					 newargs.append(tostr(paraInfo[1]))


	if "optato" in paraConf:
		for paraInfo in paraConf["optato"]:
			if paraInfo[0] in kw_args:
				newargs.append(tostr(paraInfo[1]))
				newargs.append(tostr(kw_args[paraInfo[0]]))

	if "ato" in paraConf:
		for paraInfo in paraConf["ato"]:
			if paraInfo[0] in kw_args:
				newargs.append(tostr(kw_args[paraInfo[0]]))
			elif paraInfo[1] == 1:
				raise Exception("not find necceary parameter")



	if isinstance(extpara,dict) and "ext" in paraConf:
		for paraInfo in paraConf["ext"]:
			if paraInfo[0] in kw_args:
				extpara[paraInfo[1]] = tostr(kw_args[paraInfo[0]])
	
	return newargs


def lcm (*args,**kw_args):
	"""
	.. _lcm-label:

	Linear time Closed itemset Miner
	
	:type type: 文字列
	:type i: ファイル名
	:type o: ファイル名
	:type sup: 0以上の整数
	:type K: 0以上の整数
	:type l: 0以上の整数
	:type u: 0以上の整数
	:type w: ファイル名
	:type a: float(0以上1以下)
	:type A: float(0以上1以下)
	:type r: float(0以上1以下)
	:type R: float(0以上1以下)
	:type c: ファイル名
	:type C: ファイル名
	:type item: 0以上の整数
	:type so: ファイル名
	:type separator: 文字
	:param type: 列挙するアイテム集合のタイプ(F|C|M)、及び出力形式(詳細は後述)【必須】
	:param i: トランザクションファイル名【必須】
	:param o: 出力ファイル名【必須】
	:param sup: 最小サポート
	:param U: 最大サポート
	:param K: 頻出な上位のアイテム集合のみ出力する
	:param l: アイム集合のサイズの下限値
	:param u: アイム集合のサイズの上限値
	:param w: トランザクションの重みファイル名
	:param a: 相関ルールマイニングを実施しconfidenceの下限値を与える
	:param A: 相関ルールマイニングを実施しconfidenceの上限値を与える
	:param r: 相関ルールマイニングを実施しrelational confidenceの下限値を与える
	:param R: 相関ルールマイニングを実施しrelational confidenceの上限値を与える
	:param c: read item constraint file
	:param C: read item un-constraint file
	:param item: find association rule for item [num]
	:param so: 標準出力の内容を指定のファイルに出力する	
	:param separator: 出力時のアイテムの区切り文字を指定する

	f=,F=[ratio]
		output itemsets with frequency no less/greater than [ratio] times the frequency given 
		by product of the probability of each item appearance
	p=,P=[num]
		output itemset only if (frequency)/(abusolute frequency) is no less/no greater than [num]
	n=,N=[num]
		output itemset only if its negative frequency is no less/no greater than [num] 
		(negative frequency is the sum of weights of transactions having negative weights)
	opos=,Opos=[num]
		output itemset only if its positive frequency is no less/no greater than [num] 
		(positive frequency is the sum of weights of transactions having positive weights)
	m,M=[filename]
		read/write item permutation from/to file [filename]
	stop=[num]
		stop after outputting [num] solutions
	separator=[char] 
		give the separator of the numbers in the output
	Q=[filename]
		replace the output numbers according to the permutation table given by [filename]
	# the 1st letter of input-filename cannot be '-'.
		a
	
	アイテム集合のタイプとして以下の３つから一つを選ぶ【必須】。
	
	* F:頻出アイテム集合
	* C:飽和アイテム集合
	* M:極大アイテム集合
	
	出力形式として以下の組み合わせで指定する。
	
	* f:アイテム集合の後ろに出現件数を出力する。
	* Q:アイテム集合の前に出現件数を出力する。
	* I:アイテム集合の次の行に出現集合(出現するトランザクション行番号)を出力する。
	* t:トランザクションデータベースを転置する(アイテムiはi番目のトランザクションとなり、j番目のトランザクションはアイテムjとなる)。
	* A:w=でプラスとマイナスのトランザクションの重みを指定した場合、プラス/マイナスそれぞれの件数とその比を出力する。
	* P:positive-closed itemset mining
	* R:output redundant items for rule mining (usually, redundant items are removed, to be minimal, in the case of rule mining)
	* i:do not output itemset to the output file (only rules)
	* s:output confidence and item frequency by absolute values

	例1

	0〜5のアイテムについての6行のトランザクションデータベースnumTraDB1からの実行例を以下に示す。

	.. code-block:: python
		:linenos:

		>>> import os
		>>> from nysol.take.extcore import lcm 
		>>> from nysol.take.sample import Samples
		>>> smp=Samples()
		>>> iFile=smp.numTraDB1() # カレントディレクトリにサンプルのデータファイル("numTraDB1.csv")が生成される。
		>>> smp.head(iFile,top=0) # headは指定ファイルの先頭6行を表示するメソッド(top=0で全行表示)。
		file name: ./numTraDB1.csv
		2 4
		3 4 5
		0 1 3 5
		1 3 5
		0 1 3 4
		0 1 3 4 5

		>>> lcm(type="Ff_",sup=3,i=iFile,o="rsl_Ff") # 頻出アイテム集合の列挙。type=にfを指定すると出現件数を出力、_でメッセージなし
		>>> smp.head("rsl_Ff",top=0)
		file name: rsl_Ff
		(6)
		3 (5)
		1 (4)
		1 3 (4)
		4 (4)
		4 3 (3)
		5 (4)
		5 3 (4)
		5 1 (3)
		5 1 3 (3)
		0 (3)
		0 1 (3)
		0 1 3 (3)
		0 3 (3)

		>>> lcm(type="Cf_",sup=3,i=iFile,o="rsl_Cf") # 頻出飽和アイテム集合の列挙。
		>>> smp.head("rsl_Cf",top=0)
		file name: rsl_Cf
		(6)
		3 (5)
		1 3 (4)
		4 (4)
		4 3 (3)
		5 3 (4)
		5 1 3 (3)
		0 3 1 (3)

		>>> lcm(type="Mf_",sup=3,i=iFile,o="rsl_Mf") # 頻出極大アイテム集合の列挙。
		>>> smp.head("rsl_Mf",top=0)
		file name: rsl_Mf
		4 3 (3)
		5 1 3 (3)
		0 3 1 (3)

		>>> lcm(type="Ff_",sup=3,l=3,i=iFile,o="rsl_Ff_l3") # l=3でアイテム集合のサイズが3以上に限定
		>>> smp.head("rsl_Ff",top=0)
		file name: rsl_Ff_l3
		5 1 3 (3)
		0 1 3 (3)

		>>> lcm(type="FfI_",sup=3,l=3,i=iFile,o="rsl_FfI_l3") # typeにIを加えることで、アイテム集合の下に出現集合が出力される。
		>>> smp.head("rsl_FfI_l3",top=0)
		# アイテム集合{5,1,3}はトランザクションデータの2,3,5行目に出現している(先頭行は0行目と数える}。
		file name: rsl_FfI_l3
		5 1 3 (3)
		2 3 5
		0 1 3 (3)
		2 4 5

		>>> lcm(type="FftI_",sup=3,l=3,i=iFile,o="rsl_Fft_l3") # typeにtを加えることでデータベースを転置してから実行する。
		>>> smp.head("rsl_FftI_l3",top=0)
		# トランザクション{4,2,5}はアイテム{0,1,3}に出現する。
		file name: rsl_FftI_l3
		4 2 5 (3)
		0 1 3
		3 2 5 (3)
		1 3 5

		>>> lcm(type="Ff",K=4,sup=1,i=iFile,so="rsl_Ff_K") # K=4で頻出上位4番目の出現件数を出力する。メッセージの標準出力として出力されるので、typeに"_"を入れず。so=に出力ファイル名を指定する。
		trsact: ./numTraDB1.csv ,#transactions 6 ,#items 6 ,size 21 extracted database: #transactions 6 ,#items 6 ,size 21
		separated at 0

		>>> smp.head("rsl_Ff_K",top=0) # 先に実行した例 lcm(type="Ff_",sup=3,i=iFile,o="rsl_Ff") の結果から、4番目のルールは "1 3 (4)"であり、その出現件数4が出力されている。
		file name: rsl_Ff_K
		4

		>>> wFile=smp.numTraDB1w() # 重みファイルの作成
		>>> smp.head(wFile,top=0)
		# トランザクション0,2,5行目の重みが-1で、他の行の重みが1という意味。小数で指定することも可能。
		file name: ./numTraDB1w.csv
		-1
		1
		-1
		1
		1
		-1

		>>> lcm(type="FfA_",sup=1,w=wFile,i=iFile,o="rsl_FfA_w") # w=を指定すると、トランザクションの重みで件数をカウントする。typeに"A"を加えることで、マイナス重みとプラス重みの件数情報が表示される。
		>>> smp.head("rsl_Ff_w",top=0)
		# アイテム3は、1,2,3,4,5行目に含まれ、その重み合計は1(=1-1+1+1-1)となる(プラス重み件数3件、マイナス重み件数2件、プラス率は3/5=0.6)。
		# 同様にアイテム集合{4,3}は1,4,5行目に含まれ、その重み合計は1(=1+1-1)となる(プラス重み件数2件、マイナス重み件数1件、プラス率は2/3=0.6666)。
		file name: rsl_FfA_w
		3 (1) (3,2,0.6)
		4 3 (1) (2,1,0.6666)

	"""

	paraLIST={
		"saki":[("type",1)],
		"ato":[("i",1),("sup",1),("o",0)],
		"opt":[("K","-K"),("l","-l"),("u","-u"),("U","-U"),("w","-w"),("c","-c"),("C","-C"),
					("item","-i"),("a","-a"),("A","-A"),("r","-r"),("R","-R"),("f","-f"),("F","-F"),
					("p","-p"),("P","-P"),("n","-n"),("N","-N"),("opos","-o"),("Opos","-O"),
					("m","-m"),("M","-M"),("Q","-Q"),("stop","-#"),("separator","-,")],
		"ext":[("so","so")]
	}
	extpara={}
	kwd = _kwd2list(args,kw_args,paraLIST,extpara)
	if "so" in extpara:
		return lib_lcm.lcm_run(kwd,extpara["so"])
	else:
		return lib_lcm.lcm_run(kwd)
	
def lcmseq (*args, **kw_args):
	"""
LCM algorithm for enumerating all frequently appearing sequences

keyword
type=以下の組み合わせ 必須
  %:show progress
  _:no message
  +:write solutions in append mode
  F:position occurrence, C:document occurrence
  m:output extension maximal patterns only
  c:output extension closed patterns only
  f,Q:output frequency following/preceding to each output sequence
  A:output coverages for positive/negative transactions
  I(J):output ID's of transactions including each pattern, 
       if J is given, an occurrence is written in a complete stype; 
       transaction ID, starting position and ending position
  i:do not output itemset to the output file (only rules)
  s:output confidence and item frequency by absolute values
  t:transpose the input database 
    (item i will be i-th transaction, and i-th transaction will be item i)
i=[filename]　必須
  input-filename
sup=[support]　必須
  support
o=[filename] 
  output-filename
K=[num] 
  output [num] most frequent sequences
l=,u=[num] 
  output sequences with size at least/most [num]
U=[num] 
  upper bound for support(maximum support)
g=[num]
  restrict gap length of each consequtive items by [num]
G=[num]
  restrict window size of the occurrence by [num]
w=[filename]
  read weights of transactions from the file
item=[num]
  find association rule for item [num]
a=,A=[ratio]
  find association rules of confidence at least/most [ratio]
r=,R=[ratio]
  find association rules of relational confidence at least/most [ratio]
f=,F=[ratio]
  output sequences with frequency no less/greater than [ratio] times 
  the frequency given by the product of appearance probability of each item
p=,P=[num]
  output sequence only if (frequency)/(abusolute frequency) is no less/no greater than [num]
n=,N=[num]
  output sequence only if its negative frequency is no less/no greater than [num] 
  (negative frequency is the sum of weights of transactions having negative weights)
opos=,Opos=[num]
  output sequence only if its positive frequency is no less/no greater than [num] 
  (positive frequency is the sum of weights of transactions having positive weights)
s=[num]
  output itemset rule (of the form (a,b,c) => (d,e)) with confidence at least [num] 
  (only those whose frequency of the result is no less than the support)
stop=[num]
  stop after outputting [num] solutions
separator=[char]
  give the separator of the numbers in the output
Q=[filename]
  replace the output numbers according to the permutation table given by [filename]
so=[filename]
  standard output to [filename]
# the 1st letter of input-filename cannot be '-'.
	"""
	paraLIST={
		"saki":[("type",1)],
		"ato":[("i",1),("sup",1),("o",0)],
		"opt":[("K","-K"),("l","-l"),("u","-u"),("U","-U"),("g","-g"),("G","-G"),("w","-w"),
					("item","-i"),("a","-a"),("A","-A"),("r","-r"),("R","-R"),("f","-f"),("F","-F"),
					("p","-p"),("P","-P"),("n","-n"),("N","-N"),("opos","-o"),("Opos","-O"),
					("s","-s"),("Q","-Q"),("stop","-#"),("separator","-,")],
		"ext":[("so","so")]
	}

	extpara={}
	kwd = _kwd2list(args,kw_args,paraLIST,extpara)
	if "so" in extpara:
		return lib_lcmseq.lcmseq_run(kwd,extpara["so"])
	else:
		return lib_lcmseq.lcmseq_run(kwd)

def lcmseq_zero (*args, **kw_args):
	"""
LCM algorithm for enumerating all frequently appearing sequences
0アイテムを出力しないVersion

keyword
type=以下の組み合わせ 必須
  %:show progress
  _:no message
  +:write solutions in append mode
  F:position occurrence
  C:document occurrence
  m:output extension maximal patterns only
  c:output extension closed patterns only
  f,Q:output frequency following/preceding to each output sequence
  A:output coverages for positive/negative transactions
  I(J):output ID's of transactions including each pattern, 
  　　　if J is given, an occurrence is written in a complete stype; 
  　　　transaction ID, starting position and ending position
  i:do not output itemset to the output file (only rules)
  s:output confidence and item frequency by absolute values
  t:transpose the input database 
    (item i will be i-th transaction, and i-th transaction will be item i)
i=[filename]　必須
  input-filename
sup=[support]　必須
  support
o=[filename] 
  output-filename
K=[num]
  output [num] most frequent sequences
l=,u=[num]
  output sequences with size at least/most [num]
U=[num]
  upper bound for support(maximum support)
g=[num]
  restrict gap length of each consequtive items by [num]
G=[num]
  restrict window size of the occurrence by [num]
w=[filename]
  read weights of transactions from the file
item=[num]
  find association rule for item [num]
a=,A=[ratio]
  find association rules of confidence at least/most [ratio]
r=,R=[ratio]
  find association rules of relational confidence at least/most [ratio]
f=,F=[ratio]
  output sequences with frequency no less/greater than [ratio] times 
  the frequency given by the product of appearance probability of each item
p=,P=[num]
  output sequence only if (frequency)/(abusolute frequency) is no less/no greater than [num]
n=,N=[num]
  output sequence only if its negative frequency is no less/no greater than [num] 
  (negative frequency is the sum of weights of transactions having negative weights)
opos=,Opos=[num]
  output sequence only if its positive frequency is no less/no greater than [num] 
  (positive frequency is the sum of weights of transactions having positive weights)
s=[num]
  output itemset rule (of the form (a,b,c) => (d,e)) with confidence at least [num] 
  (only those whose frequency of the result is no less than the support)
stop=[num]
  stop after outputting [num] solutions
separator=[char]
  give the separator of the numbers in the output
Q=[filename]
  replace the output numbers according to the permutation table given by [filename]
so=[filename]
  standard output to [filename]
# the 1st letter of input-filename cannot be '-'.
	"""

	paraLIST={
		"saki":[("type",1)],
		"ato":[("i",1),("sup",1),("o",0)],
		"opt":[("K","-K"),("l","-l"),("u","-u"),("U","-U"),("g","-g"),("G","-G"),("w","-w"),
					("item","-i"),("a","-a"),("A","-A"),("r","-r"),("R","-R"),("f","-f"),("F","-F"),
					("p","-p"),("P","-P"),("n","-n"),("N","-N"),("opos","-o"),("Opos","-O"),
					("s","-s"),("Q","-Q"),("stop","-#"),("separator","-,")]
	}
	kwd = _kwd2list(args,kw_args,paraLIST)
	return lib_lcmseq0.lcmseq_zero_run(kwd)

def mace(*args, **kw_args):
	"""
MAximal Clique Enumerator

keyword
type=以下の組み合わせ 必須
  %:show progress
  _:no message
  +:write solutions in append mode
  C:enumerate cliques
  M:enumerate maximal cliques
  e:edge_list format
i=[filename]　必須
  input-filename
o=[filename] 
  output-filename
l=[num]
  output cliques with size at least [num]
u=[num]
  output cliques with size at most [num]
S=[num]
  stop after outputting [num] solutions
separator=[char]
  give the separator of the numbers in the output
Q=[filename] 
  replace the output numbers according to the permutation table given by [filename]
	"""

	paraLIST={
		"saki":[("type",1)],
		"ato":[("i",1),("o",0)],
		"opt":[("l","-l"),("u","-u"),("Q","-Q"),("stop","-#"),("separator","-,")]
	}

	kwd = _kwd2list(args,kw_args,paraLIST)
	return lib_mace.mace_run(kwd)


def sspc(*args, **kw_args):
	"""
Similar Set Pair Comparison

keyword
type=以下の組み合わせ 必須
  %:show progress
  _:no message
  +:write solutions in append mode
  #:count the number of similar records for each record
  i(inclusion): find pairs [ratio] of items (weighted sum) of 
                one is included in the other (1st is included in 2nd)
  I(both-inclusion): find pairs s.t. the size (weight sum) of intersection is [ratio] of both
  S:set similarity measure to |A\cap B| / max{|A|,|B|}
  s:set similarity measure to |A\cap B| / min{|A|,|B|}
  T(intersection): find pairs having common [threshld] items
  R(resemblance): find pairs s.t. |A\capB|/|A\cupB| >= [threshld]
  P(PMI): set similarity measure to log (|A\capB|*|all| / (|A|*|B|))
          where |all| is the number of all items
  C(cosign distance): find pairs s.t. inner product of their normalized vectors >= [threshld]
  f,Q:output ratio/size of pairs following/preceding to the pairs
  D:the first entry is ID, and unify the records with the same ID
  N:normalize the ID of latter sets, in -c mode
  n:do not consider a and b in the set when comparing a and b
  Y(y):output elements of each set that contribute to no similarity (y:fast with much memory use)
  1:remove duplicated items in each transaction
  t:transpose the database so that i-th transaction will be item i
  E:input column-row representation
  w:load weight of each item in each row (with E command)
i=[filename]　必須
  input-filename
th=[ratio/threshold] 必須
  ratio/threshold
o=[filename] 
  output-filename
2=[filename]
  2nd input file name
9=[th],[filename]
  write pairs satisfies 2nd threshold [th] to file [filename]
K=[num]
  output [num] pairs of most large similarities
k=[num]
  output [num] elements of most large similarities, for each element
w=[filename]
  read item weights from [filename]
W=[filename]
  read item weights in each row from [filename]
l=,u=[num]
  ignore transactions with size (weight sum) less/more than [num]
L=,U=[num]
  ignore items appearing less/more than [num]
c=[num]
  compare transactions of IDs less than num and the others 
  (if 0 is given, automatically set to the boundary of the 1st and 2nd file)
b=[num]
  ignore pairs having no common item of at least [num]th frequency
B=[num]
  ignore pairs having no common item of frequency at most [num]
T=[num]
  ignore pairs whose intersection size is less than [num]
TT=[num]
  -T with outputting intersection size to the 1st column of each line
stop=[num]
  stop after outputting [num] solutions
separator=[char]
  give the separator of the numbers in the output
Q=[filename]
  replace the output numbers according to the permutation table given by [filename]
# the 1st letter of input-filename cannot be '-'.
	"""

	paraLIST={
		"saki":[("type",1)],
		"ato":[("i",1),("th",1),("o",0)],
		"opt":[("2","-2"),("K","-K"),("k","-k"),("w","-w"),("W","-W"),("l","-l"),("u","-u"),
					("L","-L"),("U","-U"),("c","-c"),("b","-b"),("B","-B"),("T","-T"),("TT","-TT"),
					("Q","-Q"),("stop","-#"),("separator","-,")],
		"opts":[("9","-9",2)]
	}
	kwd = _kwd2list(args,kw_args,paraLIST)
	return lib_sspc.sspc_run(kwd)
	
def medset(*args, **kw_args):
	"""
compute the intersection of each set of sets

keyword
ci=[filename] 必須
  cluster-filename
si=[filename] 必須
  set-filename
th=[threshold] 必須 
  threshold
o=[filename]
  output-filename
H=True
  do not use histgram, just output the items
R=True
  do not output singleton clusters
V=True
  output ratio of appearances of all items
T=True
  clustering by connected component (read edge type file)
I=True
  find an independent set, and clustering by using the vertices in it as seeds 
  (read edge type files)
i=True
  output for each item, ratio of records including the item
t=True
  transpose the input database, (transaction will be item, and vice varsa)
l=[num]
  output clusters of size at least [num]
progress=True
  show progress
nomsg=True
  no message
# the 1st letter of input-filename cannot be '-'.
# if threshold is negative, output the items whose frequencies are no more than the threshold
	"""
	paraLIST={
		"ato":[("ci",1),("si",1),("th",1),("o",1)],
		"opt":[("l","-l")],
		"optB":[("H","-H"),("R","-R"),("V","-V"),("T","-T"),
						("I","-I"),("i","-i"),("t","-t"),
						("progress","-%"),("nomsg","-_")]
	}

	kwd = _kwd2list(args,kw_args,paraLIST)
	return lib_medset.medset_run(kwd)


def simset(*args, **kw_args):
	"""

similarity set

keyword
type=以下の組み合わせ 必須
  %:show progress
  _:no message
  +:write solutions in append mode
  =:do not remove temporal files
  @:do not execute data polishing, E:read edge list file
  i:set similarity measure to the ratio of one is included in the other
  I:set similarity measure to the ratio of both are included in the other
  S:set similarity measure to |A\cap B|/max(|A|,|B|)
  s:set similarity measure to |A\cap B|/min(|A|,|B|)
  C:set similarity measure to the cosign distance, 
    the inner product of the normalized characteristic vectors
  T:set similarity measure to the intersection size, i.e., |A\cap B|
  R:(recemblance) set similarity measure to |A\cap B|/|A\cup B|
  P(PMI): set similarity measure to log 
          (|A\capB|*|all| / (|A|*|B|)) where |all| is the number of all items
  M:output intersection of each clique, instead of IDs of its members
  v (with M): output ratio of records, including each item
  m:do not remove edges in the data polishing phase
  O:repeatedly similarity clustering until convergence
  Y:take intersection of original graph and polished graph, instead of clique mining
  1:do not remove the same items in a record (with -G)
  W:load weight of each element
  t:transpose the input database, so that each line will be considered as a record
i=[filename] 必須
  similarity-graph-filename
th_s=[threshold] 必須
  similarity-threshold
th_d=[threshold] 必須
  degree-threshold
o=[filename]  必須
  output-filename
G=[similarity],[threshold]:
  use [similarity] of [threshold] in the first phase (file is regarded as a transaction database)
k=[threshold]
  find only k-best for each record in -G option
M=[num],[x]
  merge similar cliques of similarity in [num] of recemblance (changes to 'x' by giving '-Mx')
m=[num],[x]
  take independently cliques from similar cliques of similarity in [num] of recemblance, 
  and merge the neighbors of each independent clique ('recemblance' changes to 'x' by giving '-Mx')
v=[num]
  specify majority threshold (default=0.5) 
  (if negative is given, items of frequency no more than -[num] are output)
u=[num]
  ignore vertices of degree more than [num]
l=[num]
  ignore vertices of degree less than [num]
U=[num]
  ignore transactions of size more than [num] (with -G)
L=[num]
  ignore transactions of size less than [num] (with -G)
I=[num]
  ignore items of frequency more than [num] (with -G)
i=[num]
  ignore items of frequency less than [num] (with -G)
II=,ii=[ratio]
  give thresholds by the ratio of #ransactions/#items)
T=,t=[num]
  ignore pairs whose intersection size is less than [num] (T for first phase with -G option, and t for polishing)
O=[num]
  specify the number of repetitions
9=[num]
  shrink records of similarity more than [num]
X=[num]
  multiply the weight by [num] (and trancate by 1, for C command)
x=[num]
  power the weight by [num] (and normalize, for C command)
y=[num]
  at last, remove edges with weight less than [num]
w=[filename]
  load weight of elements from the file
multi=[num]
  use multicores of [num] (compile by 'make multicore')
W=[dir]
  specify the working directory (folder). 
  The last letter of the directory has to be '/' ('\')
separator=
  give the separator of the numbers in the output
Q=
  replace the output numbers according to the permutation table given by [filename]

# the 1st letter of input-filename cannot be '-'.
if similarity-threshold is 0, skip the similarity graph construction phase
	"""

	paraLIST={
		"saki":[("type",1)],
		"ato":[("i",1),("th_s",1),("th_d",1),("o",1)],

		"opt":[("k","-k"),("v","-v"),("u","-u"),("l","-l"),("U","-U"),("L","-L"),
					("Iig","-I"),("iig","-i"),("II","-II"),("ii","-ii"),("T","-T"),("t","-t"),
					("O","-O"),("9","-9"),("X","-X"),("x","-x"),("y","-y"),
					("w","-w"),("multi","-!"),("W","-W"),("Q","-Q"),("separator","-,")],
		"opts":[("G","-G",2)],
		"addkw":[("m","-m"),("M","-M")],
	}


	kwd = _kwd2list(args,kw_args,paraLIST)
	return lib_simset.simset_run(kwd)

#(:ref:`詳細<grhfilType-label>` )

def grhfil(*args, **kw_args):
	"""
	*グラフフィルタ(graph filtering: transform/convert/extract graph/subgraph)*

	:type type: 文字列
	:type t: 整数
	:type T: 整数
	:type ideg: 整数
	:type Ideg: 整数
	:type odeg: 整数
	:type Odeg: 整数
	:type r: 実数
	:type R: 実数
	:type n: 整数
	:type X: 実数リスト
	:type w: 真偽
	:type W: 真偽
	:type d: ファイル名
	:type m: ファイル名
	:type M: ファイル名
	:type p: ファイル名
	:type Q: ファイル名
	:type separator: 文字
	:param type: 主パラメータ(:ref:`以下に詳述<grhfilMP-label>` )【必須】
	:param i: 入力ファイル名【必須】
	:param o: 出力ファイル名【必須】
	:param t: 次数が指定した値より小さいノードは出力しない【デフォルト=制約なし】
	:param T: 次数が指定した値より大きいノードは出力しない
	:param ideg: 有向グラフにおいて入次数が指定した値より小さいノードは出力しない
	:param Ideg: 有向グラフにおいて入次数が指定した値より大きいノードは出力しない
	:param odeg: 有向グラフにおいて出次数が指定した値より小さいノードは出力しない
	:param Odeg: 有向グラフにおいて出次数が指定した値より大きいノードは出力しない
	:param r: 重みが指定した値より小さいエッジは出力しない
	:param R: 重みが指定した値より大きいエッジは出力しない
	:param n: ノードの個数を与える【デフォルト=データ上の最大番号+1】
	:param w: 入力ファイルの3項目目を重みとして読み込む(3項目目がなければ0がセットされる)
	:param W: 出力ファイルの3項目目に重みを出力する
	:param d: 指定したグラフとの差異を出力する(gTypeとgFormatは同じでなければならない)
	:param dth: specify the threshold value (d=が必要)
	:param m: 指定したグラフとのintersectionを出力する(gTypeとgFormatは同じでなければならない)
	:param M: 指定したグラフとのunionを出力する(gTypeとgFormatは同じでなければならない)
	:param p: permute the vertex ID to coutinuous numbering and output the permutation table to the file
	:param Q: permute the numbers in the file according to the table 

	.. todo:: P,Qは未対応

	.. _grhfilMP-label:

	**主パラメータ**

	以下に示すパラメータを組み合わせて文字列として指定する(ex. "UeEs")。

	1) グラフタイプ【必須】
	グラフの種類を以下の5つから一つ選んで指定する。

	* B:二部グラフ
	* d:有向グラフ(入力隣接行列を変更しない)
	* D:逆向きの有向グラフ(隣接行列の下三角行列と上三角行列を入れ替える)
	* U:片方向無向グラフ(隣接行列の下三角行列を削除し、重複したエッジを削除する)
	* u:双方向無向グラフ(入力隣接行列の下三角行列と上三角行列を対称にする)

	2) グラフの書式【オプション】
	入出力ファイルのグラフの書式は、隣接行列形式がデフォルトであるが、
	それをノードペアのエッジ形式に変更することができる。

	* e: 入力ファイルをエッジ形式とする。
	* E: 出力ファイルをエッジ形式とする。

	3) その他の指定

	* s: 出力のグラフの隣接ノードを昇順に並べる。
	* S: 出力のグラフの隣接ノードを降順に並べる。
	* n: 入力データの一行目を"ノード数 エッジ数"とする。
	* N: 出力データの一行目に"ノード数 エッジ数"を出力する。
	* v: 入力データのノード番号は1から始まるものとする。
	* V: 出力データのノード番号は1からはじまるものとして出力する。
	* 0: 一列目にノード番号を出力する(出力が隣接行列形式の場合のみ)。
	* w: 入力ファイルの隣接ノードの次の数字を重みとする。
	* W: 出力ファイルの隣接ノードの次に重みを出力する。
	* %:show progress
	* _:no message
	* +:write solutions in append mode

	* v,V:node ID in read/write file starts from 1,
	* q:non-transform mode (valid with -P option)
	* 9:give weight 1 to each vertex ID (with 0)
	* 1:unify consecutive two same numbers into one

	例1) A-Fの6アイテムにつて、5件(T1-T6)のトランザクションをCSVファイルに保存し、
	TraDBを構築する例。

	.. code-block:: python
		:linenos:

		>>> from nysol.take.extcore import grhfil
		>>> from nysol.take.sample import Samples
		>>> smp=Samples()
		>>> fName=smp.numGraph1()
		>>> smp.head(fName)
		fName: ./numGraph1.csv
		0 1
		0 2
		0 4
		1 2
		1 4
		1 6

		>>> grhfil(type="ueE_",i=fName,o="xxa")
		>>> smp.head("xxa")
	"""

	paraLIST={
		"saki":[("type",1)],
		"ato":[("i",1),("o",1)],
		"opt":[("t","-t"),("T","-T"),("ideg","-i"),("Ideg","-I"),
					("odeg","-o"),("Odeg","-O"),("r","-r"),("r","-R"),("n","-n"),
					("w","-w"),("W","-W"),("p","-p"),("Q","-Q"),("separator","-,")],
		"opts":[("X","-X",2)],
		"addkw":[("d","-d"),("m","-m"),("M","-M")],
		"optato":[("dth","-d")]
	}

	kw_args2=kw_args

	"""
	gType=kw_args["gType"]
	giFormat=kw_args["giFormat"]
	goFormat=kw_args["goFormat"]
	if giFormat=="e":
		mainArg+="e"
	if goFormat=="e":
		mainArg+="E"
	kw_args2["type"]=mainArg
	"""

	kwd = _kwd2list(args,kw_args,paraLIST)
	return lib_grhfil.grhfil_run(kwd)

def lcmtrans(inf,ptn,outf):
	"""
	lcmの結果をcsvへと変換する
	"""
	return lib_lcmtarns.lcmtrans_run(inf,ptn,outf)


def mmaketra(**kw_args):
	"""
	csvからtraデータへと変換する
	"""
	return lib_mmaketra.mmaketra_run(**kw_args)
	
	



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/
import os
import errno
import shutil

def _makedirs27(name, mode=0o777, exist_ok=False):
	"""makedirs(name [, mode=0o777][, exist_ok=False])
	Super-mkdir; create a leaf directory and all intermediate ones.  Works like
	mkdir, except that any intermediate path segment (not just the rightmost)
	will be created if it does not exist. If the target directory already
	exists, raise an OSError if exist_ok is False. Otherwise no exception is
	raised.  This is recursive.
	"""
	head, tail = os.path.split(name)
	if not tail:
		head, tail = os.path.split(head)
	if head and tail and not os.path.exists(head):
		try:
			_makedirs27(head, exist_ok=exist_ok)
		except IOError as err:
			if err.errno != errno.ENOENT: # ENOENT: no such file or directory
				raise
			# Defeats race condition when another thread created the path
			pass
			cdir = curdir
			if isinstance(tail, bytes):
				cdir = bytes(curdir, 'ASCII')
			if tail == cdir:           # xxx/newdir/. exists if xxx/newdir exists
				return
	try:
		os.mkdir(name, mode)
	except OSError:
		# Cannot rely on checking for EEXIST, since the operating system
		# could give priority to other errors like EACCES or EROFS
		if not exist_ok or not os.path.isdir(name):
			raise


def mkDir(dname,rm=False):
	import sys
	if sys.version_info.major >= 3:

		try:
			if os.path.exists(dname):
				if rm :
					shutil.rmtree(dname)
					os.makedirs(dname)
			
			else:
				os.makedirs(dname)

		except FileExistsError:
			pass

	else:

		try:
			if os.path.exists(dname):
				if rm :
					shutil.rmtree(dname)
					_makedirs27(dname)
			
			else:
				_makedirs27(dname)

		except OSError as err:
			if err.errno != errno.EEXIST:
				raise




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Excmd(NysolMOD_CORE):
	_kwd ,_inkwd,_outkwd  = n_core.getparalist("cmd",3)
	_disabled_ouputlist = True

	def __init__(self,*args, **kw_args) :
		super(Nysol_Excmd,self).__init__("cmd",nutil.arg2dict(args,kw_args,Nysol_Excmd._kwd))


def cmd(self,*args, **kw_args):

	return Nysol_Excmd(nutil.arg2dict(args,kw_args,Nysol_Excmd._kwd)).addPre(self)

setattr(NysolMOD_CORE, "cmd", cmd)
